@@ -38,9 +38,13 @@ typedef struct AVIStream {
     int scale;
     int rate;    
     int sample_size; /* audio only data */
+    int start;
     
     int new_frame_offset; /* temporary storage (used during seek) */
     int cum_len; /* temporary storage (used during seek) */
+    
+    int prefix;                       ///< normally 'd'<<8 + 'c' or 'w'<<8 + 'b'
+    int prefix_count;
 } AVIStream;
 
 typedef struct {
@@ -48,9 +52,12 @@ typedef struct {
     int64_t  movi_end;
     offset_t movi_list;
     int index_loaded;
+    int is_odml;
     DVDemuxContext* dv_demux;
 } AVIContext;
 
+static int avi_load_index(AVFormatContext *s);
+
 #ifdef DEBUG
 static void print_tag(const char *str, unsigned int tag, int size)
 {
@@ -92,8 +99,6 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
     AVIStream *ast;
     int xan_video = 0;  /* hack to support Xan A/V */
 
-    av_set_pts_info(s, 64, 1, AV_TIME_BASE);
-
     if (get_riff(avi, pb) < 0)
         return -1;
 
@@ -119,13 +124,18 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
 #endif
             if (tag1 == MKTAG('m', 'o', 'v', 'i')) {
                 avi->movi_list = url_ftell(pb) - 4;
-                avi->movi_end = avi->movi_list + size;
+                if(size) avi->movi_end = avi->movi_list + size;
+                else     avi->movi_end = url_filesize(url_fileno(pb));
 #ifdef DEBUG
                 printf("movi end=%Lx\n", avi->movi_end);
 #endif
                 goto end_of_header;
             }
             break;
+        case MKTAG('d', 'm', 'l', 'h'):
+	    avi->is_odml = 1;
+	    url_fskip(pb, size + (size & 1));
+	    break;
         case MKTAG('a', 'v', 'i', 'h'):
 	    /* avi header */
             /* using frame_period is bad idea */
@@ -138,6 +148,7 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
                 st = av_new_stream(s, i);
                 if (!st)
                     goto fail;
+
                 ast = av_mallocz(sizeof(AVIStream));
                 if (!ast)
                     goto fail;
@@ -150,6 +161,9 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
             stream_index++;
             tag1 = get_le32(pb);
             handler = get_le32(pb); /* codec tag */
+#ifdef DEBUG
+        print_tag("strh", tag1, -1);
+#endif
             switch(tag1) {
             case MKTAG('i', 'a', 'v', 's'):
 	    case MKTAG('i', 'v', 'a', 's'):
@@ -165,14 +179,19 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
 		    handler != MKTAG('d', 'v', 's', 'l'))
 	           goto fail;
                 
+		ast = s->streams[0]->priv_data;
 		av_freep(&s->streams[0]->codec.extradata);
 		av_freep(&s->streams[0]);
 		s->nb_streams = 0;
 	        avi->dv_demux = dv_init_demux(s);
 		if (!avi->dv_demux)
 		    goto fail;
+		s->streams[0]->priv_data = ast;
+		url_fskip(pb, 3 * 4);
+		ast->scale = get_le32(pb);
+		ast->rate = get_le32(pb);
 	        stream_index = s->nb_streams - 1;
-		url_fskip(pb, size - 8);
+		url_fskip(pb, size - 7*4);
 		break;
 	    case MKTAG('v', 'i', 'd', 's'):
                 codec_type = CODEC_TYPE_VIDEO;
@@ -184,6 +203,7 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
 
                 st = s->streams[stream_index];
                 ast = st->priv_data;
+                st->codec.stream_codec_tag= handler;
                 
                 get_le32(pb); /* flags */
                 get_le16(pb); /* priority */
@@ -202,14 +222,15 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
                 }
                 ast->rate = rate;
                 ast->scale = scale;
+                av_set_pts_info(st, 64, scale, rate);
                 st->codec.frame_rate = rate;
                 st->codec.frame_rate_base = scale;
                 get_le32(pb); /* start */
                 nb_frames = get_le32(pb);
                 st->start_time = 0;
-                st->duration = (double)nb_frames * 
-                    st->codec.frame_rate_base * AV_TIME_BASE / 
-                    st->codec.frame_rate;
+                st->duration = av_rescale(nb_frames,
+                    st->codec.frame_rate_base * AV_TIME_BASE,
+                    st->codec.frame_rate);
 		url_fskip(pb, size - 9 * 4);
                 break;
             case MKTAG('a', 'u', 'd', 's'):
@@ -231,18 +252,31 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
                     get_le32(pb); /* initial frame */
                     ast->scale = get_le32(pb); /* scale */
                     ast->rate = get_le32(pb);
-                    get_le32(pb); /* start */
+                    av_set_pts_info(st, 64, ast->scale, ast->rate);
+                    ast->start= get_le32(pb); /* start */
                     length = get_le32(pb); /* length, in samples or bytes */
                     get_le32(pb); /* buffer size */
                     get_le32(pb); /* quality */
                     ast->sample_size = get_le32(pb); /* sample ssize */
+//av_log(NULL, AV_LOG_DEBUG, "%d %d %d %d\n", ast->scale, ast->rate, ast->sample_size, ast->start);
                     st->start_time = 0;
                     if (ast->rate != 0)
                         st->duration = (int64_t)length * AV_TIME_BASE / ast->rate;
                     url_fskip(pb, size - 12 * 4);
                 }
                 break;
+            case MKTAG('t', 'x', 't', 's'):
+                //FIXME 
+                codec_type = CODEC_TYPE_DATA; //CODEC_TYPE_SUB ?  FIXME
+                url_fskip(pb, size - 8);
+                break;
+            case MKTAG('p', 'a', 'd', 's'):
+                codec_type = CODEC_TYPE_UNKNOWN;
+                url_fskip(pb, size - 8);
+                stream_index--;
+                break;
             default:
+                av_log(s, AV_LOG_ERROR, "unknown stream type %X\n", tag1);
                 goto fail;
             }
             break;
@@ -311,6 +345,9 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
                         st->codec.codec_id = CODEC_ID_XAN_DPCM;
                     break;
                 default:
+                    st->codec.codec_type = CODEC_TYPE_DATA;
+                    st->codec.codec_id= CODEC_ID_NONE;
+                    st->codec.codec_tag= 0;
                     url_fskip(pb, size);
                     break;
                 }
@@ -334,6 +371,10 @@ static int avi_read_header(AVFormatContext *s, AVFormatParameters *ap)
         return -1;
     }
 
+    assert(!avi->index_loaded);
+    avi_load_index(s);
+    avi->index_loaded = 1;
+ 
     return 0;
 }
 
@@ -341,33 +382,27 @@ static int avi_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
     AVIContext *avi = s->priv_data;
     ByteIOContext *pb = &s->pb;
-    int n, d[8], size, i;
+    int n, d[8], size;
+    offset_t i, sync;
     void* dstr;
-
-    memset(d, -1, sizeof(int)*8);
    
     if (avi->dv_demux) {
         size = dv_get_packet(avi->dv_demux, pkt);
 	if (size >= 0)
 	    return size;
     }
-        
-    for(i=url_ftell(pb); !url_feof(pb); i++) {
+
+resync:
+    memset(d, -1, sizeof(int)*8);
+    for(i=sync=url_ftell(pb); !url_feof(pb); i++) {
         int j;
 
-	if (i >= avi->movi_end) { /* Let's see if it's an OpenDML AVI */
-	    uint32_t tag, size, tag2;
-	    url_fskip(pb, avi->riff_end - url_ftell(pb));
-	    if (get_riff(avi, pb) < 0)
-	        return -1;
-	    
-	    tag = get_le32(pb);
-	    size = get_le32(pb);
-	    tag2 = get_le32(pb);
-	    if (tag == MKTAG('L','I','S','T') && tag2 == MKTAG('m','o','v','i'))
-	        avi->movi_end = url_ftell(pb) + size - 4; 
-	    else
-	        return -1;
+	if (i >= avi->movi_end) {
+	    if (avi->is_odml) {
+		url_fskip(pb, avi->riff_end - i);
+	        avi->riff_end = avi->movi_end = url_filesize(url_fileno(pb));
+	    } else
+	        break;
 	}
 
         for(j=0; j<7; j++)
@@ -376,28 +411,52 @@ static int avi_read_packet(AVFormatContext *s, AVPacket *pkt)
         
         size= d[4] + (d[5]<<8) + (d[6]<<16) + (d[7]<<24);
         
-        //parse ix##
-        n= (d[2] - '0') * 10 + (d[3] - '0');
         if(    d[2] >= '0' && d[2] <= '9'
-            && d[3] >= '0' && d[3] <= '9'
-            && d[0] == 'i' && d[1] == 'x'
-            && n < s->nb_streams
-            && i + size <= avi->movi_end){
-            
+            && d[3] >= '0' && d[3] <= '9'){
+            n= (d[2] - '0') * 10 + (d[3] - '0');
+        }else{
+            n= 100; //invalid stream id
+        }
+//av_log(NULL, AV_LOG_DEBUG, "%X %X %X %X %X %X %X %X %lld %d %d\n", d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], i, size, n);
+        if(i + size > avi->movi_end || d[0]<0)
+            continue;
+        
+        //parse ix##
+        if(  (d[0] == 'i' && d[1] == 'x' && n < s->nb_streams)
+	//parse JUNK
+           ||(d[0] == 'J' && d[1] == 'U' && d[2] == 'N' && d[3] == 'K')){
             url_fskip(pb, size);
+//av_log(NULL, AV_LOG_DEBUG, "SKIP\n");
+            goto resync;
+        }
+
+        if(    d[0] >= '0' && d[0] <= '9'
+            && d[1] >= '0' && d[1] <= '9'){
+            n= (d[0] - '0') * 10 + (d[1] - '0');
+        }else{
+            n= 100; //invalid stream id
         }
         
         //parse ##dc/##wb
-        n= (d[0] - '0') * 10 + (d[1] - '0');
-        if(    d[0] >= '0' && d[0] <= '9'
-            && d[1] >= '0' && d[1] <= '9'
-            && ((d[2] == 'd' && d[3] == 'c') || 
-	        (d[2] == 'w' && d[3] == 'b') || 
-		(d[2] == 'd' && d[3] == 'b') ||
-		(d[2] == '_' && d[3] == '_'))
-            && n < s->nb_streams
-            && i + size <= avi->movi_end) {
-        
+        if(n < s->nb_streams){
+          AVStream *st;
+          AVIStream *ast;
+          st = s->streams[n];
+          ast = st->priv_data;
+
+          if(   ((ast->prefix_count<5 || sync+9 > i) && d[2]<128 && d[3]<128) || 
+                d[2]*256+d[3] == ast->prefix /*||
+                (d[2] == 'd' && d[3] == 'c') || 
+	        (d[2] == 'w' && d[3] == 'b')*/) {
+
+//av_log(NULL, AV_LOG_DEBUG, "OK\n");
+            if(d[2]*256+d[3] == ast->prefix)
+                ast->prefix_count++;
+            else{
+                ast->prefix= d[2]*256+d[3];
+                ast->prefix_count= 0;
+            }
+
             av_new_packet(pkt, size);
             get_buffer(pb, pkt->data, size);
             if (size & 1) {
@@ -412,13 +471,12 @@ static int avi_read_packet(AVFormatContext *s, AVPacket *pkt)
 		pkt->destruct = dstr;
                 pkt->flags |= PKT_FLAG_KEY;
 	    } else {
-                AVStream *st;
-                AVIStream *ast;
-                st = s->streams[n];
-                ast = st->priv_data;
-
                 /* XXX: how to handle B frames in avi ? */
-                pkt->pts = ((int64_t)ast->frame_offset * ast->scale* AV_TIME_BASE) / ast->rate;
+                pkt->dts = ast->frame_offset;
+//                pkt->dts += ast->start;
+                if(ast->sample_size)
+                    pkt->dts /= ast->sample_size;
+//av_log(NULL, AV_LOG_DEBUG, "dts:%Ld offset:%d %d/%d smpl_siz:%d base:%d st:%d size:%d\n", pkt->dts, ast->frame_offset, ast->scale, ast->rate, ast->sample_size, AV_TIME_BASE, n, size);
                 pkt->stream_index = n;
                 /* FIXME: We really should read index for that */
                 if (st->codec.codec_type == CODEC_TYPE_VIDEO) {
@@ -430,15 +488,46 @@ static int avi_read_packet(AVFormatContext *s, AVPacket *pkt)
                            are key frames */
                         pkt->flags |= PKT_FLAG_KEY;
                     }
-                    ast->frame_offset++;
                 } else {
-                    ast->frame_offset += pkt->size;
                     pkt->flags |= PKT_FLAG_KEY; 
                 }
+                if(ast->sample_size)
+                    ast->frame_offset += pkt->size;
+                else
+                    ast->frame_offset++;
 	    }
             return size;
+          }
         }
+        /* palette changed chunk */
+        if (   d[0] >= '0' && d[0] <= '9'
+            && d[1] >= '0' && d[1] <= '9'
+            && ((d[2] == 'p' && d[3] == 'c'))
+            && n < s->nb_streams && i + size <= avi->movi_end) {
+
+            AVStream *st;
+            int first, clr, flags, k, p;
+
+            st = s->streams[n];
+
+            first = get_byte(pb);
+            clr = get_byte(pb);
+            flags = get_le16(pb);
+            p = 4;
+            for (k = first; k < clr + first; k++) {
+                int r, g, b;
+                r = get_byte(pb);
+                g = get_byte(pb);
+                b = get_byte(pb);
+                    get_byte(pb);
+                st->codec.palctrl->palette[k] = b + (g << 8) + (r << 16);
+            }
+            st->codec.palctrl->palette_changed = 1;
+            goto resync;
+        }
+
     }
+
     return -1;
 }
 
@@ -495,7 +584,8 @@ static int avi_load_index(AVFormatContext *s)
     AVIContext *avi = s->priv_data;
     ByteIOContext *pb = &s->pb;
     uint32_t tag, size;
-
+    offset_t pos= url_ftell(pb);
+    
     url_fseek(pb, avi->movi_end, SEEK_SET);
 #ifdef DEBUG_SEEK
     printf("movi_end=0x%llx\n", avi->movi_end);
@@ -528,6 +618,7 @@ static int avi_load_index(AVFormatContext *s)
         }
     }
  the_end:
+    url_fseek(pb, pos, SEEK_SET);
     return 0;
 }
 
@@ -557,7 +648,7 @@ static int locate_frame_in_index(AVIIndexEntry *entries,
     return m;
 }
 
-static int avi_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp)
+static int avi_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp, int flags)
 {
     AVIContext *avi = s->priv_data;
     AVStream *st;
@@ -586,8 +677,7 @@ static int avi_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp
         return -1;
     ast = st->priv_data;
     /* compute the frame number */
-    frame_number = (timestamp * ast->rate) /
-        (ast->scale * (int64_t)AV_TIME_BASE);
+    frame_number = timestamp;
 #ifdef DEBUG_SEEK
     printf("timestamp=%0.3f nb_indexes=%d frame_number=%d\n", 
            (double)timestamp / AV_TIME_BASE,
@@ -627,7 +717,7 @@ static int avi_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp
             if ((j  + 1) < ast->nb_index_entries)
                 j++;
             /* extract the current frame number */
-            if (st->codec.codec_type == CODEC_TYPE_VIDEO)           
+            if (ast->sample_size==0)
                 ast->new_frame_offset = j;
             else
                 ast->new_frame_offset = ast->index_entries[j].cum_len;
@@ -644,6 +734,8 @@ static int avi_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp
                ast->frame_offset);
 #endif
     }
+    if (avi->dv_demux)
+        dv_flush_audio_packets(avi->dv_demux);
     /* do the seek */
     pos += avi->movi_list;
     url_fseek(&s->pb, pos, SEEK_SET);
@@ -658,8 +750,10 @@ static int avi_read_close(AVFormatContext *s)
     for(i=0;i<s->nb_streams;i++) {
         AVStream *st = s->streams[i];
         AVIStream *ast = st->priv_data;
-        av_free(ast->index_entries);
-        av_free(ast);
+        if(ast){
+            av_free(ast->index_entries);
+            av_free(ast);
+        }
         av_free(st->codec.extradata);
         av_free(st->codec.palctrl);
     }
