@@ -3,7 +3,7 @@
  * Declaration and extern access to global program object.
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001, 2002 VideoLAN
- * $Id: main.h,v 1.56 2003/10/29 01:33:27 gbazin Exp $
+ * $Id: main.h 8911 2004-10-04 16:03:30Z gbazin $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -11,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -55,6 +55,9 @@ struct libvlc_t
     module_bank_t *        p_module_bank;
 
     /* Arch-specific variables */
+#if !defined( WIN32 )
+    vlc_bool_t             b_daemon;
+#endif 
 #if defined( SYS_BEOS )
     vlc_object_t *         p_appthread;
     char *                 psz_vlcpath;
@@ -65,6 +68,8 @@ struct libvlc_t
     vlc_bool_t             b_fast_mutex;
     int                    i_win9x_cv;
     char *                 psz_vlcpath;
+#elif defined( UNDER_CE )
+    char *                 psz_vlcpath;
 #endif
 };
 
