@@ -3,7 +3,7 @@
  * This header provides a portable threads implementation.
  *****************************************************************************
  * Copyright (C) 1999, 2002 VideoLAN
- * $Id: vlc_threads_funcs.h 6961 2004-03-05 17:34:23Z sam $
+ * $Id: vlc_threads_funcs.h 8414 2004-08-11 23:34:38Z titer $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -346,153 +346,6 @@ static inline int __vlc_cond_signal( char * psz_file, int i_line,
     return i_result;
 }
 
-/*****************************************************************************
- * vlc_cond_broadcast: start all threads waiting on condition completion
- *****************************************************************************/
-/*
- * FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME
- * Only works with pthreads, st, win32
- * You need to adapt it for others
- * FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME FIXME
- */
-#define vlc_cond_broadcast( P_COND )                                        \
-    __vlc_cond_broadcast( __FILE__, __LINE__, P_COND )
-
-static inline int __vlc_cond_broadcast( char * psz_file, int i_line,
-                                        vlc_cond_t *p_condvar )
-{
-    int i_result;
-    /* In case of error : */
-    int i_thread = -1;
-    const char * psz_error = "";
-
-#if defined( PTH_INIT_IN_PTH_H )
-    i_result = ( pth_cond_notify( &p_condvar->cond, TRUE ) == FALSE );
-
-#elif defined( ST_INIT_IN_ST_H )
-    i_result = st_cond_broadcast( p_condvar->cond );
-
-#elif defined( UNDER_CE )
-    int i;
-
-    /* Release all waiting threads. */
-    for( i = p_condvar->i_waiting_threads; i > 0; i-- )
-    {
-        PulseEvent( p_condvar->event );
-    }
-    i_result = 0;
-
-#elif defined( WIN32 )
-    int i;
-
-    /* Release all waiting threads. */
-    if( !p_condvar->semaphore )
-    {
-        for( i = p_condvar->i_waiting_threads; i > 0; i-- )
-        {
-            PulseEvent( p_condvar->event );
-        }
-    }
-    else if( p_condvar->i_win9x_cv == 1 )
-    {
-        /* Wait for the gate to be open */
-        WaitForSingleObject( p_condvar->event, INFINITE );
-
-        if( p_condvar->i_waiting_threads )
-        {
-            /* Using a semaphore exposes us to a race condition. It is
-             * possible for another thread to start waiting on the semaphore
-             * just after we signaled it and thus steal the signal.
-             * We have to prevent new threads from entering the cond_wait(). */
-            ResetEvent( p_condvar->event );
-
-            /* A semaphore is used here because Win9x doesn't have
-             * SignalObjectAndWait() and thus a race condition exists
-             * during the time we release the mutex and the time we start
-             * waiting on the event (more precisely, the signal can sometimes
-             * be missed by the waiting thread if we use PulseEvent()). */
-            ReleaseSemaphore( p_condvar->semaphore,
-                              p_condvar->i_waiting_threads, 0 );
-        }
-    }
-    else
-    {
-        if( p_condvar->i_waiting_threads )
-        {
-            ReleaseSemaphore( p_condvar->semaphore,
-                              p_condvar->i_waiting_threads, 0 );
-
-            /* Wait for the last thread to be awakened */
-            WaitForSingleObject( p_condvar->event, INFINITE );
-        }
-    }
-    i_result = 0;
-
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    if( p_condvar == NULL )
-    {
-        i_result = B_BAD_VALUE;
-    }
-    else if( p_condvar->init < 2000 )
-    {
-        i_result = B_NO_INIT;
-    }
-    else
-    {
-        while( p_condvar->thread != -1 )
-        {
-            thread_info info;
-            if( get_thread_info(p_condvar->thread, &info) == B_BAD_VALUE )
-            {
-                return 0;
-            }
-
-            if( info.state != B_THREAD_SUSPENDED )
-            {
-                /* The  waiting thread is not suspended so it could
-                 * have been interrupted beetwen the unlock and the
-                 * suspend_thread line. That is why we sleep a little
-                 * before retesting p_condver->thread. */
-                snooze( 10000 );
-            }
-            else
-            {
-                /* Ok, we have to wake up that thread */
-                resume_thread( p_condvar->thread );
-                return 0;
-            }
-        }
-        i_result = 0;
-    }
-
-#elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
-    i_result = pthread_cond_broadcast( &p_condvar->cond );
-    if ( i_result )
-    {
-        i_thread = (int)pthread_self();
-        psz_error = strerror(i_result);
-    }
-
-#elif defined( HAVE_CTHREADS_H )
-    /* condition_signal() */
-    if ( p_condvar->queue.head || p_condvar->implications )
-    {
-        cond_signal( (condition_t)p_condvar );
-    }
-    i_result = 0;
-
-#endif
-
-    if( i_result )
-    {
-        msg_Err( p_condvar->p_this,
-                 "thread %d: cond_broadcast failed at %s:%d (%d:%s)",
-                 i_thread, psz_file, i_line, i_result, psz_error );
-    }
-
-    return i_result;
-}
-
 /*****************************************************************************
  * vlc_cond_wait: wait until condition completion
  *****************************************************************************/
