@@ -1,8 +1,8 @@
 /*****************************************************************************
- * win32_specific.h: Win32 specific features 
+ * win32_specific.h: Win32 specific features
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: win32_specific.h,v 1.2 2002/04/02 23:43:57 gbazin Exp $
+ * $Id: win32_specific.h 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -11,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,20 +22,3 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-
-typedef BOOL (WINAPI *SIGNALOBJECTANDWAIT)( HANDLE, HANDLE, DWORD, BOOL );
-
-/*****************************************************************************
- * main_sys_t: system specific descriptor
- *****************************************************************************
- * This structure is a system specific descriptor. It describes the Win32
- * properties of the program.
- *****************************************************************************/
-typedef struct main_sys_s
-{
-    SIGNALOBJECTANDWAIT SignalObjectAndWait;
-    boolean_t b_fast_pthread;
-
-} main_sys_t;
