@@ -38,47 +38,54 @@ void E_(Close)   ( vlc_object_t * );
  * Module descriptor
  *****************************************************************************/
 
-#define FREQ_TEXT N_("satellite default transponder frequency")
+#define DEMUX_TEXT N_("Demux number")
+#define DEMUX_LONGTEXT ""
+
+#define TUNER_TEXT N_("Tuner number")
+#define TUNER_LONGTEXT ""
+
+#define FREQ_TEXT N_("Satellite default transponder frequency (kHz)")
 #define FREQ_LONGTEXT ""
 
-#define POL_TEXT N_("satellite default transponder polarization")
+#define POL_TEXT N_("Satellite default transponder polarization")
 #define POL_LONGTEXT ""
 
-#define FEC_TEXT N_("satellite default transponder FEC")
+#define FEC_TEXT N_("Satellite default transponder FEC")
 #define FEC_LONGTEXT ""
 
-#define SRATE_TEXT N_("satellite default transponder symbol rate")
+#define SRATE_TEXT N_("Satellite default transponder symbol rate (kHz)")
 #define SRATE_LONGTEXT ""
 
-#define DISEQC_TEXT N_("use diseqc with antenna")
+#define DISEQC_TEXT N_("Use diseqc with antenna")
 #define DISEQC_LONGTEXT ""
 
-#define LNB_LOF1_TEXT N_("antenna lnb_lof1 (kHz)")
+#define LNB_LOF1_TEXT N_("Antenna lnb_lof1 (kHz)")
 #define LNB_LOF1_LONGTEXT ""
 
-#define LNB_LOF2_TEXT N_("antenna lnb_lof2 (kHz)")
+#define LNB_LOF2_TEXT N_("Antenna lnb_lof2 (kHz)")
 #define LNB_LOF2_LONGTEXT ""
 
-#define LNB_SLOF_TEXT N_("antenna lnb_slof (kHz)")
+#define LNB_SLOF_TEXT N_("Antenna lnb_slof (kHz)")
 #define LNB_SLOF_LONGTEXT ""
 
 vlc_module_begin();
-    add_category_hint( N_("Input"), NULL, VLC_FALSE );
-        add_integer( "frequency", 11954, NULL, FREQ_TEXT, FREQ_LONGTEXT,
-                     VLC_FALSE );
-        add_integer( "polarization", 0, NULL, POL_TEXT, POL_LONGTEXT,
-                     VLC_FALSE );
-        add_integer( "fec", 3, NULL, FEC_TEXT, FEC_LONGTEXT, VLC_FALSE );
-        add_integer( "symbol-rate", 27500, NULL, SRATE_TEXT, SRATE_LONGTEXT,
-                     VLC_FALSE );
-        add_bool( "diseqc", 0, NULL, DISEQC_TEXT, DISEQC_LONGTEXT, VLC_FALSE );
-        add_integer( "lnb-lof1", 10000, NULL,
-                     LNB_LOF1_TEXT, LNB_LOF1_LONGTEXT, VLC_FALSE );
-        add_integer( "lnb-lof2", 10000, NULL,
-                     LNB_LOF2_TEXT, LNB_LOF2_LONGTEXT, VLC_FALSE );
-        add_integer( "lnb-slof", 11700, NULL,
-                     LNB_SLOF_TEXT, LNB_SLOF_LONGTEXT, VLC_FALSE );
-    set_description( _("satellite input") );
+    set_description( _("Satellite input") );
+
+    add_integer( "dvb-dmx", 0, NULL, DEMUX_TEXT, DEMUX_LONGTEXT, VLC_FALSE );
+    add_integer( "dvb-tuner", 0, NULL, TUNER_TEXT, TUNER_LONGTEXT, VLC_FALSE );
+    add_integer( "frequency", 0, NULL, FREQ_TEXT, FREQ_LONGTEXT, VLC_FALSE );
+    add_integer( "polarization", 0, NULL, POL_TEXT, POL_LONGTEXT, VLC_FALSE );
+    add_integer( "fec", 3, NULL, FEC_TEXT, FEC_LONGTEXT, VLC_FALSE );
+    add_integer( "symbol-rate", 27500000, NULL, SRATE_TEXT, SRATE_LONGTEXT,
+                  VLC_FALSE );
+    add_bool( "diseqc", 0, NULL, DISEQC_TEXT, DISEQC_LONGTEXT, VLC_TRUE );
+    add_integer( "lnb-lof1", 10000000, NULL,
+                 LNB_LOF1_TEXT, LNB_LOF1_LONGTEXT, VLC_TRUE );
+    add_integer( "lnb-lof2", 10000000, NULL,
+                 LNB_LOF2_TEXT, LNB_LOF2_LONGTEXT, VLC_TRUE );
+    add_integer( "lnb-slof", 11700000, NULL,
+                 LNB_SLOF_TEXT, LNB_SLOF_LONGTEXT, VLC_TRUE );
+
     set_capability( "access", 0 );
     add_shortcut( "sat" );
     set_callbacks( E_(Open), E_(Close) );
