@@ -2,7 +2,7 @@
  * encoder.c: video and audio encoder using the ffmpeg library
  *****************************************************************************
  * Copyright (C) 1999-2004 VideoLAN
- * $Id: encoder.c 9086 2004-10-30 15:27:28Z gbazin $
+ * $Id: encoder.c 9156 2004-11-05 14:57:53Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -351,9 +351,8 @@ int E_(OpenEncoder)( vlc_object_t *p_this )
                    VOUT_ASPECT_FACTOR, 1 << 30 /* something big */ );
         av_reduce( &p_context->sample_aspect_ratio.num,
                    &p_context->sample_aspect_ratio.den,
-                   i_aspect_num *
-                   (int64_t)p_context->height / p_context->width,
-                   i_aspect_den, 1 << 30 /* something big */ );
+                   i_aspect_num * (int64_t)p_context->height,
+                   i_aspect_den * (int64_t)p_context->width, 1 << 30 );
 #else
         p_context->aspect_ratio = ((float)p_enc->fmt_in.video.i_aspect) /
             VOUT_ASPECT_FACTOR;
@@ -609,11 +608,11 @@ static block_t *EncodeVideo( encoder_t *p_enc, picture_t *p_pict )
     /* Let ffmpeg select the frame type */
     frame.pict_type = 0;
 
-    frame.repeat_pict = p_pict->i_nb_fields;
+    frame.repeat_pict = 2 - p_pict->i_nb_fields;
 
 #if LIBAVCODEC_BUILD >= 4685
     frame.interlaced_frame = !p_pict->b_progressive;
-    frame.top_field_first = p_pict->b_top_field_first;
+    frame.top_field_first = !!p_pict->b_top_field_first;
 #endif
 
 #if LIBAVCODEC_BUILD < 4702
