@@ -2,7 +2,7 @@
  * ffmpeg.c: video decoder using ffmpeg library
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: ffmpeg.c 8994 2004-10-15 08:58:16Z gbazin $
+ * $Id: ffmpeg.c 9307 2004-11-13 13:09:42Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -350,7 +350,9 @@ static struct
 
     /* Packed YUV formats */
     { VLC_FOURCC('Y','U','Y','2'), PIX_FMT_YUV422 },
-    { VLC_FOURCC('U','Y','V','Y'), PIX_FMT_YUV422 },
+#if LIBAVCODEC_BUILD >= 4720
+    { VLC_FOURCC('U','Y','V','Y'), PIX_FMT_UYVY422 },
+#endif
 
     /* Packed RGB formats */
     { VLC_FOURCC('R','V','1','5'), PIX_FMT_RGB555 },
