@@ -2,7 +2,7 @@
  * theora.c: theora decoder module making use of libtheora.
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: theora.c 7543 2004-04-28 20:02:13Z gbazin $
+ * $Id: theora.c 8813 2004-09-26 20:17:50Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -26,6 +26,7 @@
  *****************************************************************************/
 #include <vlc/vlc.h>
 #include <vlc/decoder.h>
+#include <vlc/input.h>
 #include <vlc/sout.h>
 
 #include <ogg/ogg.h>
@@ -66,6 +67,7 @@ static int  OpenPacketizer( vlc_object_t * );
 static void CloseDecoder  ( vlc_object_t * );
 
 static void *DecodeBlock  ( decoder_t *, block_t ** );
+static int  ProcessHeaders( decoder_t * );
 static void *ProcessPacket ( decoder_t *, ogg_packet *, block_t ** );
 
 static picture_t *DecodePacket( decoder_t *, ogg_packet * );
@@ -75,7 +77,6 @@ static void theora_CopyPicture( decoder_t *, picture_t *, yuv_buffer * );
 
 static int  OpenEncoder( vlc_object_t *p_this );
 static void CloseEncoder( vlc_object_t *p_this );
-static block_t *Headers( encoder_t *p_enc );
 static block_t *Encode( encoder_t *p_enc, picture_t *p_pict );
 
 /*****************************************************************************
@@ -194,80 +195,179 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
     oggpacket.e_o_s = 0;
     oggpacket.packetno = 0;
 
-    if( p_sys->i_headers == 0 )
+    /* Check for headers */
+    if( p_sys->i_headers == 0 && p_dec->fmt_in.i_extra )
     {
-        /* Take care of the initial Theora header */
+        /* Headers already available as extra data */
+        p_sys->i_headers = 3;
+    }
+    else if( oggpacket.bytes && p_sys->i_headers < 3 )
+    {
+        /* Backup headers as extra data */
+        uint8_t *p_extra;
 
-        oggpacket.b_o_s = 1; /* yes this actually is a b_o_s packet :) */
-        if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
-        {
-            msg_Err( p_dec, "This bitstream does not contain Theora "
-                     "video data." );
-            block_Release( p_block );
-            return NULL;
-        }
+        p_dec->fmt_in.p_extra =
+            realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra +
+                     oggpacket.bytes + 2 );
+        p_extra = p_dec->fmt_in.p_extra + p_dec->fmt_in.i_extra;
+        *(p_extra++) = oggpacket.bytes >> 8;
+        *(p_extra++) = oggpacket.bytes & 0xFF;
+
+        memcpy( p_extra, oggpacket.packet, oggpacket.bytes );
+        p_dec->fmt_in.i_extra += oggpacket.bytes + 2;
+
+        block_Release( *pp_block );
         p_sys->i_headers++;
-
-        /* Set output properties */
-        p_dec->fmt_out.video.i_width = p_sys->ti.width;
-        p_dec->fmt_out.video.i_height = p_sys->ti.height;
-
-        if( p_sys->ti.aspect_denominator && p_sys->ti.aspect_numerator )
-            p_dec->fmt_out.video.i_aspect = ((int64_t)VOUT_ASPECT_FACTOR) *
-                p_sys->ti.aspect_numerator / p_sys->ti.aspect_denominator;
-        else
-            p_dec->fmt_out.video.i_aspect = VOUT_ASPECT_FACTOR *
-                p_sys->ti.frame_width / p_sys->ti.frame_height;
-
-        msg_Dbg( p_dec, "%dx%d %.02f fps video, frame content "
-                 "is %dx%d with offset (%d,%d).",
-                 p_sys->ti.width, p_sys->ti.height,
-                 (double)p_sys->ti.fps_numerator/p_sys->ti.fps_denominator,
-                 p_sys->ti.frame_width, p_sys->ti.frame_height,
-                 p_sys->ti.offset_x, p_sys->ti.offset_y );
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
+        return NULL;
     }
 
-    if( p_sys->i_headers == 1 )
+    if( p_sys->i_headers == 3 )
     {
-        /* The next packet in order is the comments header */
-        if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
+        if( ProcessHeaders( p_dec ) != VLC_SUCCESS )
         {
-            msg_Err( p_dec, "2nd Theora header is corrupted." );
+            p_sys->i_headers = 0;
+            p_dec->fmt_in.i_extra = 0;
+            block_Release( *pp_block );
             return NULL;
         }
-        p_sys->i_headers++;
-
-        ParseTheoraComments( p_dec );
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
-    }
-
-    if( p_sys->i_headers == 2 )
-    {
-        /* The next packet in order is the codebooks header
-           We need to watch out that this packet is not missing as a
-           missing or corrupted header is fatal. */
-        if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
-        {
-            msg_Err( p_dec, "3rd Theora header is corrupted." );
-            return NULL;
-        }
-        p_sys->i_headers++;
-
-        if( !p_sys->b_packetizer )
-        {
-            /* We have all the headers, initialize decoder */
-            theora_decode_init( &p_sys->td, &p_sys->ti );
-        }
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
+        else p_sys->i_headers++;
     }
 
     return ProcessPacket( p_dec, &oggpacket, pp_block );
 }
 
+/*****************************************************************************
+ * ProcessHeaders: process Theora headers.
+ *****************************************************************************/
+static int ProcessHeaders( decoder_t *p_dec )
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+    ogg_packet oggpacket;
+    uint8_t *p_extra;
+    int i_extra;
+
+    if( !p_dec->fmt_in.i_extra ) return VLC_EGENERIC;
+
+    oggpacket.granulepos = -1;
+    oggpacket.b_o_s = 1; /* yes this actually is a b_o_s packet :) */
+    oggpacket.e_o_s = 0;
+    oggpacket.packetno = 0;
+    p_extra = p_dec->fmt_in.p_extra;
+    i_extra = p_dec->fmt_in.i_extra;
+
+    /* Take care of the initial Vorbis header */
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    p_extra += oggpacket.bytes;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "This bitstream does not contain Theora video data" );
+        return VLC_EGENERIC;
+    }
+
+    /* Set output properties */
+    p_dec->fmt_out.video.i_width = p_sys->ti.width;
+    p_dec->fmt_out.video.i_height = p_sys->ti.height;
+    if( p_sys->ti.frame_width && p_sys->ti.frame_height )
+    {
+        p_dec->fmt_out.video.i_width = p_sys->ti.frame_width;
+        p_dec->fmt_out.video.i_height = p_sys->ti.frame_height;
+    }
+
+    if( p_sys->ti.aspect_denominator && p_sys->ti.aspect_numerator )
+    {
+        p_dec->fmt_out.video.i_aspect = ((int64_t)VOUT_ASPECT_FACTOR) *
+            ( p_sys->ti.aspect_numerator * p_sys->ti.width ) /
+            ( p_sys->ti.aspect_denominator * p_sys->ti.height );
+    }
+    else
+    {
+        p_dec->fmt_out.video.i_aspect = VOUT_ASPECT_FACTOR *
+            p_sys->ti.frame_width / p_sys->ti.frame_height;
+    }
+
+    if( p_sys->ti.fps_numerator > 0 && p_sys->ti.fps_denominator > 0 )
+    {
+        p_dec->fmt_out.video.i_frame_rate = p_sys->ti.fps_numerator;
+        p_dec->fmt_out.video.i_frame_rate_base = p_sys->ti.fps_denominator;
+    }
+
+    msg_Dbg( p_dec, "%dx%d %.02f fps video, frame content "
+             "is %dx%d with offset (%d,%d)",
+             p_sys->ti.width, p_sys->ti.height,
+             (double)p_sys->ti.fps_numerator/p_sys->ti.fps_denominator,
+             p_sys->ti.frame_width, p_sys->ti.frame_height,
+             p_sys->ti.offset_x, p_sys->ti.offset_y );
+
+    /* The next packet in order is the comments header */
+    oggpacket.b_o_s = 0;
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    p_extra += oggpacket.bytes;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    /* The next packet in order is the comments header */
+    if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "2nd Theora header is corrupted" );
+        return VLC_EGENERIC;
+    }
+
+    ParseTheoraComments( p_dec );
+
+    /* The next packet in order is the codebooks header
+     * We need to watch out that this packet is not missing as a
+     * missing or corrupted header is fatal. */
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    /* The next packet in order is the codebooks header
+     * We need to watch out that this packet is not missing as a
+     * missing or corrupted header is fatal */
+    if( theora_decode_header( &p_sys->ti, &p_sys->tc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "3rd Theora header is corrupted" );
+        return VLC_EGENERIC;
+    }
+
+    if( !p_sys->b_packetizer )
+    {
+        /* We have all the headers, initialize decoder */
+        theora_decode_init( &p_sys->td, &p_sys->ti );
+    }
+    else
+    {
+        p_dec->fmt_out.i_extra = p_dec->fmt_in.i_extra;
+        p_dec->fmt_out.p_extra =
+            realloc( p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
+        memcpy( p_dec->fmt_out.p_extra,
+                p_dec->fmt_in.p_extra, p_dec->fmt_out.i_extra );
+    }
+
+    return VLC_SUCCESS;
+}
+
 /*****************************************************************************
  * ProcessPacket: processes a theora packet.
  *****************************************************************************/
@@ -284,6 +384,8 @@ static void *ProcessPacket( decoder_t *p_dec, ogg_packet *p_oggpacket,
         p_sys->i_pts = p_block->i_pts;
     }
 
+    *pp_block = NULL; /* To avoid being fed the same packet again */
+
     if( p_sys->b_packetizer )
     {
         /* Date management */
@@ -303,11 +405,7 @@ static void *ProcessPacket( decoder_t *p_dec, ogg_packet *p_oggpacket,
         else
             p_buf = NULL;
 
-        if( p_block )
-        {
-            block_Release( p_block );
-            *pp_block = NULL;
-        }
+        if( p_block ) block_Release( p_block );
     }
 
     /* Date management */
@@ -348,9 +446,11 @@ static picture_t *DecodePacket( decoder_t *p_dec, ogg_packet *p_oggpacket )
 static void ParseTheoraComments( decoder_t *p_dec )
 {
     input_thread_t *p_input = (input_thread_t *)p_dec->p_parent;
-
-    int i = 0;
     char *psz_name, *psz_value, *psz_comment;
+    int i = 0;
+
+    if( p_input->i_object_type != VLC_OBJECT_INPUT ) return;
+
     while ( i < p_dec->p_sys->tc.comments )
     {
         psz_comment = strdup( p_dec->p_sys->tc.user_comments[i] );
@@ -415,7 +515,7 @@ static void theora_CopyPicture( decoder_t *p_dec, picture_t *p_pic,
 
         p_src += (i_src_yoffset * i_src_stride + i_src_yoffset);
 
-        for( i_line = 0; i_line < p_pic->p[i_plane].i_lines; i_line++ )
+        for( i_line = 0; i_line < p_pic->p[i_plane].i_visible_lines; i_line++ )
         {
             p_dec->p_vlc->pf_memcpy( p_dst, p_src, i_width );
             p_src += i_src_stride;
@@ -441,10 +541,7 @@ struct encoder_sys_t
     theora_comment   tc;                            /* theora comment header */
     theora_state     td;                   /* theora bitstream user comments */
 
-    /*
-     * Common properties
-     */
-    mtime_t i_pts;
+    int i_width, i_height;
 };
 
 /*****************************************************************************
@@ -454,8 +551,10 @@ static int OpenEncoder( vlc_object_t *p_this )
 {
     encoder_t *p_enc = (encoder_t *)p_this;
     encoder_sys_t *p_sys = p_enc->p_sys;
+    ogg_packet header;
+    uint8_t *p_extra;
     vlc_value_t val;
-    int i_quality;
+    int i_quality, i;
 
     if( p_enc->fmt_out.i_codec != VLC_FOURCC('t','h','e','o') &&
         !p_enc->b_force )
@@ -463,15 +562,6 @@ static int OpenEncoder( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    if( p_enc->fmt_in.video.i_width % 16 ||
-        p_enc->fmt_in.video.i_height % 16 )
-    {
-        msg_Err( p_enc, "Theora video encoding requires dimensions which are "
-                 "multiples of 16. Which is not the case here (%dx%d).",
-                 p_enc->fmt_in.video.i_width, p_enc->fmt_in.video.i_height );
-        return VLC_EGENERIC;
-    }
-
     /* Allocate the memory needed to store the decoder's structure */
     if( ( p_sys = (encoder_sys_t *)malloc(sizeof(encoder_sys_t)) ) == NULL )
     {
@@ -480,12 +570,11 @@ static int OpenEncoder( vlc_object_t *p_this )
     }
     p_enc->p_sys = p_sys;
 
-    p_enc->pf_header = Headers;
     p_enc->pf_encode_video = Encode;
     p_enc->fmt_in.i_codec = VLC_FOURCC('I','4','2','0');
     p_enc->fmt_out.i_codec = VLC_FOURCC('t','h','e','o');
 
-    sout_ParseCfg( p_enc, ENC_CFG_PREFIX, ppsz_enc_options, p_enc->p_cfg );
+    sout_CfgParse( p_enc, ENC_CFG_PREFIX, ppsz_enc_options, p_enc->p_cfg );
 
     var_Get( p_enc, ENC_CFG_PREFIX "quality", &val );
     i_quality = val.i_int;
@@ -496,10 +585,26 @@ static int OpenEncoder( vlc_object_t *p_this )
 
     p_sys->ti.width = p_enc->fmt_in.video.i_width;
     p_sys->ti.height = p_enc->fmt_in.video.i_height;
+
+    if( p_sys->ti.width % 16 || p_sys->ti.height % 16 )
+    {
+        /* Pictures from the transcoder should always have a pitch
+         * which is a multiple of 16 */
+        p_sys->ti.width = (p_sys->ti.width + 15) >> 4 << 4;
+        p_sys->ti.height = (p_sys->ti.height + 15) >> 4 << 4;
+
+        msg_Dbg( p_enc, "padding video from %dx%d to %dx%d",
+                 p_enc->fmt_in.video.i_width, p_enc->fmt_in.video.i_height,
+                 p_sys->ti.width, p_sys->ti.height );
+    }
+
     p_sys->ti.frame_width = p_enc->fmt_in.video.i_width;
     p_sys->ti.frame_height = p_enc->fmt_in.video.i_height;
     p_sys->ti.offset_x = 0 /*frame_x_offset*/;
-    p_sys->ti.offset_y = 0/*frame_y_offset*/;
+    p_sys->ti.offset_y = 0 /*frame_y_offset*/;
+
+    p_sys->i_width = p_sys->ti.width;
+    p_sys->i_height = p_sys->ti.height;
 
     if( !p_enc->fmt_in.video.i_frame_rate ||
         !p_enc->fmt_in.video.i_frame_rate_base )
@@ -515,8 +620,14 @@ static int OpenEncoder( vlc_object_t *p_this )
 
     if( p_enc->fmt_in.video.i_aspect )
     {
-        p_sys->ti.aspect_numerator = p_enc->fmt_in.video.i_aspect;
-        p_sys->ti.aspect_denominator = VOUT_ASPECT_FACTOR;
+        int64_t i_num, i_den;
+        int i_dst_num, i_dst_den;
+
+        i_num = p_enc->fmt_in.video.i_aspect * (int64_t)p_sys->ti.height;
+        i_den = VOUT_ASPECT_FACTOR * p_sys->ti.width;
+        vlc_reduce( &i_dst_num, &i_dst_den, i_num, i_den, 0 );
+        p_sys->ti.aspect_numerator = i_dst_num;
+        p_sys->ti.aspect_denominator = i_dst_den;
     }
     else
     {
@@ -541,54 +652,28 @@ static int OpenEncoder( vlc_object_t *p_this )
     theora_info_clear( &p_sys->ti );
     theora_comment_init( &p_sys->tc );
 
-    p_sys->b_headers = VLC_FALSE;
-
-    return VLC_SUCCESS;
-}
-
-/****************************************************************************
- * Encode: the whole thing
- ****************************************************************************
- * This function spits out ogg packets.
- ****************************************************************************/
-static block_t *Headers( encoder_t *p_enc )
-{
-    encoder_sys_t *p_sys = p_enc->p_sys;
-    block_t *p_chain = NULL;
-
-    /* Create theora headers */
-    if( !p_sys->b_headers )
+    /* Create and store headers */
+    p_enc->fmt_out.i_extra = 3 * 2;
+    for( i = 0; i < 3; i++ )
     {
-        ogg_packet oggpackets;
-        int i;
-        block_t *p_block;
+        if( i == 0 ) theora_encode_header( &p_sys->td, &header );
+        else if( i == 1 ) theora_encode_comment( &p_sys->tc, &header );
+        else if( i == 2 ) theora_encode_tables( &p_sys->td, &header );
 
-        /* Ogg packet to block */
-        for( i = 0; i < 3; i++ )
-        {
-            switch( i )
-            {
-            case 0:
-                theora_encode_header( &p_sys->td, &oggpackets );
-                break;
-            case 1:
-                theora_encode_comment( &p_sys->tc, &oggpackets );
-                break;
-            case 2:
-                theora_encode_tables( &p_sys->td, &oggpackets );
-                break;
-            }
+        p_enc->fmt_out.p_extra =
+            realloc( p_enc->fmt_out.p_extra,
+                     p_enc->fmt_out.i_extra + header.bytes );
+        p_extra = p_enc->fmt_out.p_extra;
+        p_extra += p_enc->fmt_out.i_extra + (i-3)*2;
+        p_enc->fmt_out.i_extra += header.bytes;
 
-            p_block = block_New( p_enc, oggpackets.bytes );
-            memcpy( p_block->p_buffer, oggpackets.packet, oggpackets.bytes );
-            p_block->i_dts = p_block->i_pts = p_block->i_length = 0;
-            block_ChainAppend( &p_chain, p_block );
-        }
+        *(p_extra++) = header.bytes >> 8;
+        *(p_extra++) = header.bytes & 0xFF;
 
-        p_sys->b_headers = VLC_TRUE;
+        memcpy( p_extra, header.packet, header.bytes );
     }
 
-    return p_chain;
+    return VLC_SUCCESS;
 }
 
 /****************************************************************************
@@ -602,16 +687,70 @@ static block_t *Encode( encoder_t *p_enc, picture_t *p_pict )
     ogg_packet oggpacket;
     block_t *p_block;
     yuv_buffer yuv;
+    int i;
+
+    /* Sanity check */
+    if( p_pict->p[0].i_pitch < (int)p_sys->i_width ||
+        p_pict->p[0].i_lines < (int)p_sys->i_height )
+    {
+        msg_Warn( p_enc, "frame is smaller than encoding size"
+                  "(%ix%i->%ix%i) -> dropping frame",
+                  p_pict->p[0].i_pitch, p_pict->p[0].i_lines,
+                  p_sys->i_width, p_sys->i_height );
+        return NULL;
+    }
+
+    /* Fill padding */
+    if( p_pict->p[0].i_visible_pitch < (int)p_sys->i_width )
+    {
+        for( i = 0; i < p_sys->i_height; i++ )
+        {
+            memset( p_pict->p[0].p_pixels + i * p_pict->p[0].i_pitch +
+                    p_pict->p[0].i_visible_pitch,
+                    *( p_pict->p[0].p_pixels + i * p_pict->p[0].i_pitch +
+                       p_pict->p[0].i_visible_pitch - 1 ),
+                    p_sys->i_width - p_pict->p[0].i_visible_pitch );
+        }
+        for( i = 0; i < p_sys->i_height / 2; i++ )
+        {
+            memset( p_pict->p[1].p_pixels + i * p_pict->p[1].i_pitch +
+                    p_pict->p[1].i_visible_pitch,
+                    *( p_pict->p[1].p_pixels + i * p_pict->p[1].i_pitch +
+                       p_pict->p[1].i_visible_pitch - 1 ),
+                    p_sys->i_width / 2 - p_pict->p[1].i_visible_pitch );
+            memset( p_pict->p[2].p_pixels + i * p_pict->p[2].i_pitch +
+                    p_pict->p[2].i_visible_pitch,
+                    *( p_pict->p[2].p_pixels + i * p_pict->p[2].i_pitch +
+                       p_pict->p[2].i_visible_pitch - 1 ),
+                    p_sys->i_width / 2 - p_pict->p[2].i_visible_pitch );
+        }
+    }
+
+    if( p_pict->p[0].i_visible_lines < (int)p_sys->i_height )
+    {
+        for( i = p_pict->p[0].i_visible_lines; i < p_sys->i_height; i++ )
+        {
+            memset( p_pict->p[0].p_pixels + i * p_pict->p[0].i_pitch, 0,
+                    p_sys->i_width );
+        }
+        for( i = p_pict->p[1].i_visible_lines; i < p_sys->i_height / 2; i++ )
+        {
+            memset( p_pict->p[1].p_pixels + i * p_pict->p[1].i_pitch, 0x80,
+                    p_sys->i_width / 2 );
+            memset( p_pict->p[2].p_pixels + i * p_pict->p[2].i_pitch, 0x80,
+                    p_sys->i_width / 2 );
+        }
+    }
 
     /* Theora is a one-frame-in, one-frame-out system. Submit a frame
      * for compression and pull out the packet. */
 
-    yuv.y_width  = p_pict->p[0].i_visible_pitch;
-    yuv.y_height = p_pict->p[0].i_lines;
+    yuv.y_width  = p_sys->i_width;
+    yuv.y_height = p_sys->i_height;
     yuv.y_stride = p_pict->p[0].i_pitch;
 
-    yuv.uv_width  = p_pict->p[1].i_visible_pitch;
-    yuv.uv_height = p_pict->p[1].i_lines;
+    yuv.uv_width  = p_sys->i_width / 2;
+    yuv.uv_height = p_sys->i_height / 2;
     yuv.uv_stride = p_pict->p[1].i_pitch;
 
     yuv.y = p_pict->p[0].p_pixels;
