@@ -2,7 +2,7 @@
  * vorbis.c: vorbis decoder/encoder/packetizer module making use of libvorbis.
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: vorbis.c 7491 2004-04-25 20:48:54Z gbazin $
+ * $Id: vorbis.c 8565 2004-08-29 10:56:24Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -26,6 +26,7 @@
  *****************************************************************************/
 #include <vlc/vlc.h>
 #include <vlc/decoder.h>
+#include <vlc/input.h>
 #include <vlc/sout.h>
 
 #include <ogg/ogg.h>
@@ -100,6 +101,7 @@ static int  OpenPacketizer( vlc_object_t * );
 static void CloseDecoder  ( vlc_object_t * );
 static void *DecodeBlock  ( decoder_t *, block_t ** );
 
+static int  ProcessHeaders( decoder_t * );
 static void *ProcessPacket ( decoder_t *, ogg_packet *, block_t ** );
 
 static aout_buffer_t *DecodePacket  ( decoder_t *, ogg_packet * );
@@ -116,7 +118,6 @@ static void Interleave   ( float *, const float **, int, int );
 #ifndef MODULE_NAME_IS_tremor
 static int OpenEncoder   ( vlc_object_t * );
 static void CloseEncoder ( vlc_object_t * );
-static block_t *Headers  ( encoder_t * );
 static block_t *Encode   ( encoder_t *, aout_buffer_t * );
 #endif
 
@@ -135,6 +136,9 @@ static block_t *Encode   ( encoder_t *, aout_buffer_t * );
 #define ENC_MINBR_LONGTEXT N_( \
   "Allows you to specify a minimum bitrate in kbps. " \
   "Useful for encoding for a fixed-size channel." )
+#define ENC_CBR_TEXT N_("CBR encoding")
+#define ENC_CBR_LONGTEXT N_( \
+  "Allows you to force a constant bitrate encoding (CBR)." )
 
 vlc_module_begin();
 
@@ -158,19 +162,21 @@ vlc_module_begin();
     set_capability( "encoder", 100 );
     set_callbacks( OpenEncoder, CloseEncoder );
 
-    add_integer( ENC_CFG_PREFIX "quality", 0, NULL, ENC_QUALITY_TEXT,
+    add_integer( ENC_CFG_PREFIX "quality", 3, NULL, ENC_QUALITY_TEXT,
                  ENC_QUALITY_LONGTEXT, VLC_FALSE );
     add_integer( ENC_CFG_PREFIX "max-bitrate", 0, NULL, ENC_MAXBR_TEXT,
                  ENC_MAXBR_LONGTEXT, VLC_FALSE );
     add_integer( ENC_CFG_PREFIX "min-bitrate", 0, NULL, ENC_MINBR_TEXT,
                  ENC_MINBR_LONGTEXT, VLC_FALSE );
+    add_bool( ENC_CFG_PREFIX "cbr", 0, NULL, ENC_CBR_TEXT,
+                 ENC_CBR_LONGTEXT, VLC_FALSE );
 #endif
 
 vlc_module_end();
 
 #ifndef MODULE_NAME_IS_tremor
 static const char *ppsz_enc_options[] = {
-    "quality", "max-bitrate", "min-bitrate", NULL
+    "quality", "max-bitrate", "min-bitrate", "cbr", NULL
 };
 #endif
 
@@ -268,82 +274,155 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
     oggpacket.e_o_s = 0;
     oggpacket.packetno = 0;
 
-    if( p_sys->i_headers == 0 )
+    /* Check for headers */
+    if( p_sys->i_headers == 0 && p_dec->fmt_in.i_extra )
     {
-        /* Take care of the initial Vorbis header */
+        /* Headers already available as extra data */
+        p_sys->i_headers = 3;
+    }
+    else if( oggpacket.bytes && p_sys->i_headers < 3 )
+    {
+        /* Backup headers as extra data */
+        uint8_t *p_extra;
 
-        oggpacket.b_o_s = 1; /* yes this actually is a b_o_s packet :) */
-        if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc,
-                                       &oggpacket ) < 0 )
-        {
-            msg_Err( p_dec, "this bitstream does not contain Vorbis "
-                     "audio data.");
-            block_Release( *pp_block );
-            return NULL;
-        }
+        p_dec->fmt_in.p_extra =
+            realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra +
+                     oggpacket.bytes + 2 );
+        p_extra = p_dec->fmt_in.p_extra + p_dec->fmt_in.i_extra;
+        *(p_extra++) = oggpacket.bytes >> 8;
+        *(p_extra++) = oggpacket.bytes & 0xFF;
+
+        memcpy( p_extra, oggpacket.packet, oggpacket.bytes );
+        p_dec->fmt_in.i_extra += oggpacket.bytes + 2;
+
+        block_Release( *pp_block );
         p_sys->i_headers++;
-
-        /* Setup the format */
-        p_dec->fmt_out.audio.i_rate     = p_sys->vi.rate;
-        p_dec->fmt_out.audio.i_channels = p_sys->vi.channels;
-        p_dec->fmt_out.audio.i_physical_channels =
-            p_dec->fmt_out.audio.i_original_channels =
-                pi_channels_maps[p_sys->vi.channels];
-        p_dec->fmt_out.i_bitrate = p_sys->vi.bitrate_nominal;
-
-        aout_DateInit( &p_sys->end_date, p_sys->vi.rate );
-
-        msg_Dbg( p_dec, "channels:%d samplerate:%ld bitrate:%ld",
-                 p_sys->vi.channels, p_sys->vi.rate,
-                 p_sys->vi.bitrate_nominal );
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
+        return NULL;
     }
 
-    if( p_sys->i_headers == 1 )
+    if( p_sys->i_headers == 3 )
     {
-        /* The next packet in order is the comments header */
-        if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc, &oggpacket )
-            < 0 )
+        if( ProcessHeaders( p_dec ) != VLC_SUCCESS )
         {
-            msg_Err( p_dec, "2nd Vorbis header is corrupted" );
+            p_sys->i_headers = 0;
+            p_dec->fmt_in.i_extra = 0;
             block_Release( *pp_block );
             return NULL;
         }
-        p_sys->i_headers++;
-
-        ParseVorbisComments( p_dec );
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
-    }
-
-    if( p_sys->i_headers == 2 )
-    {
-        /* The next packet in order is the codebooks header
-           We need to watch out that this packet is not missing as a
-           missing or corrupted header is fatal. */
-        if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc, &oggpacket )
-            < 0 )
-        {
-            msg_Err( p_dec, "3rd Vorbis header is corrupted" );
-            block_Release( *pp_block );
-            return NULL;
-        }
-        p_sys->i_headers++;
-
-        if( !p_sys->b_packetizer )
-        {
-            /* Initialize the Vorbis packet->PCM decoder */
-            vorbis_synthesis_init( &p_sys->vd, &p_sys->vi );
-            vorbis_block_init( &p_sys->vd, &p_sys->vb );
-        }
-
-        return ProcessPacket( p_dec, &oggpacket, pp_block );
+        else p_sys->i_headers++;
     }
 
     return ProcessPacket( p_dec, &oggpacket, pp_block );
 }
 
+/*****************************************************************************
+ * ProcessHeaders: process Vorbis headers.
+ *****************************************************************************/
+static int ProcessHeaders( decoder_t *p_dec )
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+    ogg_packet oggpacket;
+    uint8_t *p_extra;
+    int i_extra;
+
+    if( !p_dec->fmt_in.i_extra ) return VLC_EGENERIC;
+
+    oggpacket.granulepos = -1;
+    oggpacket.b_o_s = 1; /* yes this actually is a b_o_s packet :) */
+    oggpacket.e_o_s = 0;
+    oggpacket.packetno = 0;
+    p_extra = p_dec->fmt_in.p_extra;
+    i_extra = p_dec->fmt_in.i_extra;
+
+    /* Take care of the initial Vorbis header */
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    p_extra += oggpacket.bytes;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "this bitstream does not contain Vorbis audio data");
+        return VLC_EGENERIC;
+    }
+
+    /* Setup the format */
+    p_dec->fmt_out.audio.i_rate     = p_sys->vi.rate;
+    p_dec->fmt_out.audio.i_channels = p_sys->vi.channels;
+    p_dec->fmt_out.audio.i_physical_channels =
+        p_dec->fmt_out.audio.i_original_channels =
+            pi_channels_maps[p_sys->vi.channels];
+    p_dec->fmt_out.i_bitrate = p_sys->vi.bitrate_nominal;
+
+    aout_DateInit( &p_sys->end_date, p_sys->vi.rate );
+    aout_DateSet( &p_sys->end_date, 0 );
+
+    msg_Dbg( p_dec, "channels:%d samplerate:%ld bitrate:%ld",
+             p_sys->vi.channels, p_sys->vi.rate, p_sys->vi.bitrate_nominal );
+
+    /* The next packet in order is the comments header */
+    oggpacket.b_o_s = 0;
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    p_extra += oggpacket.bytes;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "2nd Vorbis header is corrupted" );
+        return VLC_EGENERIC;
+    }
+    ParseVorbisComments( p_dec );
+
+    /* The next packet in order is the codebooks header
+     * We need to watch out that this packet is not missing as a
+     * missing or corrupted header is fatal. */
+    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes |= (*(p_extra++) & 0xFF);
+    oggpacket.packet = p_extra;
+    i_extra -= (oggpacket.bytes + 2);
+    if( i_extra < 0 )
+    {
+        msg_Err( p_dec, "header data corrupted");
+        return VLC_EGENERIC;
+    }
+
+    if( vorbis_synthesis_headerin( &p_sys->vi, &p_sys->vc, &oggpacket ) < 0 )
+    {
+        msg_Err( p_dec, "3rd Vorbis header is corrupted" );
+        return VLC_EGENERIC;
+    }
+
+    if( !p_sys->b_packetizer )
+    {
+        /* Initialize the Vorbis packet->PCM decoder */
+        vorbis_synthesis_init( &p_sys->vd, &p_sys->vi );
+        vorbis_block_init( &p_sys->vd, &p_sys->vb );
+    }
+    else
+    {
+        p_dec->fmt_out.i_extra = p_dec->fmt_in.i_extra;
+        p_dec->fmt_out.p_extra =
+            realloc( p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
+        memcpy( p_dec->fmt_out.p_extra,
+                p_dec->fmt_in.p_extra, p_dec->fmt_out.i_extra );
+    }
+
+    return VLC_SUCCESS;
+}
+
 /*****************************************************************************
  * ProcessPacket: processes a Vorbis packet.
  *****************************************************************************/
@@ -382,10 +461,7 @@ static void *ProcessPacket( decoder_t *p_dec, ogg_packet *p_oggpacket,
         else
             p_aout_buffer = NULL;
 
-        if( p_block )
-        {
-            block_Release( p_block );
-        }
+        if( p_block ) block_Release( p_block );
         return p_aout_buffer;
     }
 }
@@ -479,9 +555,12 @@ static block_t *SendPacket( decoder_t *p_dec, ogg_packet *p_oggpacket,
 static void ParseVorbisComments( decoder_t *p_dec )
 {
     input_thread_t *p_input = (input_thread_t *)p_dec->p_parent;
-    int i = 0;
     char *psz_name, *psz_value, *psz_comment;
-    while ( i < p_dec->p_sys->vc.comments )
+    int i = 0;
+
+    if( p_input->i_object_type != VLC_OBJECT_INPUT ) return;
+
+    while( i < p_dec->p_sys->vc.comments )
     {
         psz_comment = strdup( p_dec->p_sys->vc.user_comments[i] );
         if( !psz_comment )
@@ -548,15 +627,10 @@ static void CloseDecoder( vlc_object_t *p_this )
 #if defined(HAVE_VORBIS_VORBISENC_H) && !defined(MODULE_NAME_IS_tremor)
 
 /*****************************************************************************
- * encoder_sys_t : theora encoder descriptor
+ * encoder_sys_t : vorbis encoder descriptor
  *****************************************************************************/
 struct encoder_sys_t
 {
-    /*
-     * Input properties
-     */
-    int i_headers;
-
     /*
      * Vorbis properties
      */
@@ -585,8 +659,10 @@ static int OpenEncoder( vlc_object_t *p_this )
 {
     encoder_t *p_enc = (encoder_t *)p_this;
     encoder_sys_t *p_sys;
-    int i_quality, i_min_bitrate, i_max_bitrate;
+    int i_quality, i_min_bitrate, i_max_bitrate, i;
+    ogg_packet header[3];
     vlc_value_t val;
+    uint8_t *p_extra;
 
     if( p_enc->fmt_out.i_codec != VLC_FOURCC('v','o','r','b') &&
         !p_enc->b_force )
@@ -602,17 +678,18 @@ static int OpenEncoder( vlc_object_t *p_this )
     }
     p_enc->p_sys = p_sys;
 
-    p_enc->pf_header = Headers;
     p_enc->pf_encode_audio = Encode;
     p_enc->fmt_in.i_codec = VLC_FOURCC('f','l','3','2');
     p_enc->fmt_out.i_codec = VLC_FOURCC('v','o','r','b');
 
-    sout_ParseCfg( p_enc, ENC_CFG_PREFIX, ppsz_enc_options, p_enc->p_cfg );
+    sout_CfgParse( p_enc, ENC_CFG_PREFIX, ppsz_enc_options, p_enc->p_cfg );
 
     var_Get( p_enc, ENC_CFG_PREFIX "quality", &val );
     i_quality = val.i_int;
     if( i_quality > 10 ) i_quality = 10;
     if( i_quality < 0 ) i_quality = 0;
+    var_Get( p_enc, ENC_CFG_PREFIX "cbr", &val );
+    if( val.b_bool ) i_quality = 0;
     var_Get( p_enc, ENC_CFG_PREFIX "max-bitrate", &val );
     i_max_bitrate = val.i_int;
     var_Get( p_enc, ENC_CFG_PREFIX "min-bitrate", &val );
@@ -670,56 +747,36 @@ static int OpenEncoder( vlc_object_t *p_this )
 
     vorbis_encode_setup_init( &p_sys->vi );
 
-    /* add a comment */
+    /* Add a comment */
     vorbis_comment_init( &p_sys->vc);
     vorbis_comment_add_tag( &p_sys->vc, "ENCODER", "VLC media player");
 
-    /* set up the analysis state and auxiliary encoding storage */
+    /* Set up the analysis state and auxiliary encoding storage */
     vorbis_analysis_init( &p_sys->vd, &p_sys->vi );
     vorbis_block_init( &p_sys->vd, &p_sys->vb );
 
+    /* Create and store headers */
+    vorbis_analysis_headerout( &p_sys->vd, &p_sys->vc,
+                               &header[0], &header[1], &header[2]);
+    p_enc->fmt_out.i_extra = 3 * 2 + header[0].bytes +
+       header[1].bytes + header[2].bytes;
+    p_extra = p_enc->fmt_out.p_extra = malloc( p_enc->fmt_out.i_extra );
+    for( i = 0; i < 3; i++ )
+    {
+        *(p_extra++) = header[i].bytes >> 8;
+        *(p_extra++) = header[i].bytes & 0xFF;
+        memcpy( p_extra, header[i].packet, header[i].bytes );
+        p_extra += header[i].bytes;
+    }
+
     p_sys->i_channels = p_enc->fmt_in.audio.i_channels;
     p_sys->i_last_block_size = 0;
     p_sys->i_samples_delay = 0;
-    p_sys->i_headers = 0;
     p_sys->i_pts = 0;
 
     return VLC_SUCCESS;
 }
 
-/****************************************************************************
- * Encode: the whole thing
- ****************************************************************************
- * This function spits out ogg packets.
- ****************************************************************************/
-static block_t *Headers( encoder_t *p_enc )
-{
-    encoder_sys_t *p_sys = p_enc->p_sys;
-    block_t *p_block, *p_chain = NULL;
-
-    /* Create theora headers */
-    if( !p_sys->i_headers )
-    {
-        ogg_packet header[3];
-        int i;
-
-        vorbis_analysis_headerout( &p_sys->vd, &p_sys->vc,
-                                   &header[0], &header[1], &header[2]);
-        for( i = 0; i < 3; i++ )
-        {
-            p_block = block_New( p_enc, header[i].bytes );
-            memcpy( p_block->p_buffer, header[i].packet, header[i].bytes );
-
-            p_block->i_dts = p_block->i_pts = p_block->i_length = 0;
-
-            block_ChainAppend( &p_chain, p_block );
-        }
-        p_sys->i_headers = 3;
-    }
-
-    return p_chain;
-}
-
 /****************************************************************************
  * Encode: the whole thing
  ****************************************************************************
@@ -747,8 +804,8 @@ static block_t *Encode( encoder_t *p_enc, aout_buffer_t *p_aout_buf )
     {
         for( j = 0 ; j < p_aout_buf->i_nb_samples ; j++ )
         {
-            buffer[i][j]= ((float)( ((int16_t *)p_aout_buf->p_buffer )
-                                    [j * p_sys->i_channels + i ] )) / 32768.f;
+            buffer[i][j]= ((float *)p_aout_buf->p_buffer)
+                                    [j * p_sys->i_channels + i ];
         }
     }
 
@@ -790,7 +847,7 @@ static block_t *Encode( encoder_t *p_enc, aout_buffer_t *p_aout_buf )
 }
 
 /*****************************************************************************
- * CloseEncoder: theora encoder destruction
+ * CloseEncoder: vorbis encoder destruction
  *****************************************************************************/
 static void CloseEncoder( vlc_object_t *p_this )
 {
