@@ -2,7 +2,7 @@
  * menus.cpp : wxWindows plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2004 VideoLAN
- * $Id: menus.cpp 7737 2004-05-21 10:48:33Z gbazin $
+ * $Id: menus.cpp 8715 2004-09-16 14:31:14Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -91,7 +91,14 @@ enum
     OpenDisc_Event,
     OpenNet_Event,
     OpenCapture_Event,
+    MediaInfo_Event,
+    Messages_Event,
     Preferences_Event,
+    Play_Event,
+    Pause_Event,
+    Previous_Event,
+    Next_Event,
+    Stop_Event,
     FirstAutoGenerated_Event = wxID_HIGHEST + 1999,
     SettingsMenu_Events = wxID_HIGHEST + 5000,
     AudioMenu_Events = wxID_HIGHEST + 2000,
@@ -110,6 +117,8 @@ BEGIN_EVENT_TABLE(MenuEvtHandler, wxEvtHandler)
     EVT_MENU(OpenDisc_Event, MenuEvtHandler::OnShowDialog)
     EVT_MENU(OpenNet_Event, MenuEvtHandler::OnShowDialog)
     EVT_MENU(OpenCapture_Event, MenuEvtHandler::OnShowDialog)
+    EVT_MENU(MediaInfo_Event, MenuEvtHandler::OnShowDialog)
+    EVT_MENU(Messages_Event, MenuEvtHandler::OnShowDialog)
     EVT_MENU(Preferences_Event, MenuEvtHandler::OnShowDialog)
     EVT_MENU(-1, MenuEvtHandler::OnMenuEvent)
 END_EVENT_TABLE()
@@ -125,78 +134,29 @@ wxMenu *OpenStreamMenu( intf_thread_t *p_intf )
     return menu;
 }
 
+wxMenu *MiscMenu( intf_thread_t *p_intf )
+{
+    wxMenu *menu = new wxMenu;
+    menu->Append( MediaInfo_Event, wxU(_("Media &Info...")) );
+    menu->Append( Messages_Event, wxU(_("&Messages...")) );
+    menu->Append( Preferences_Event, wxU(_("&Preferences...")) );
+    return menu;
+}
+
 void PopupMenu( intf_thread_t *p_intf, wxWindow *p_parent,
                 const wxPoint& pos )
 {
-#define MAX_POPUP_ITEMS 35
+#define MAX_POPUP_ITEMS 45
 
-    vlc_object_t *p_object;
+    vlc_object_t *p_object, *p_input;
     char *ppsz_varnames[MAX_POPUP_ITEMS];
     int pi_objects[MAX_POPUP_ITEMS];
-    int i = 0;
+    int i = 0, i_last_separator = 0;
 
     /* Initializations */
     memset( pi_objects, 0, MAX_POPUP_ITEMS * sizeof(int) );
 
-    /* Audio menu */
-    ppsz_varnames[i++] = _("Audio menu");
-    ppsz_varnames[i++] = NULL; /* Separator */
-
-    p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                FIND_ANYWHERE );
-    if( p_object != NULL )
-    {
-        ppsz_varnames[i] = "audio-device";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "audio-channels";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "visual";
-        pi_objects[i++] = p_object->i_object_id;
-        vlc_object_release( p_object );
-    }
-
-    /* Video menu */
-    ppsz_varnames[i++] = NULL; /* Separator */
-    ppsz_varnames[i++] = _("Video menu");
-    ppsz_varnames[i++] = NULL; /* Separator */
-
-    p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                                FIND_ANYWHERE );
-    if( p_object != NULL )
-    {
-        vlc_object_t *p_dec_obj;
-
-        ppsz_varnames[i] = "fullscreen";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "zoom";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "deinterlace";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "aspect-ratio";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "crop";
-        pi_objects[i++] = p_object->i_object_id;
-        ppsz_varnames[i] = "video-on-top";
-        pi_objects[i++] = p_object->i_object_id;
-
-        p_dec_obj = (vlc_object_t *)vlc_object_find( p_object,
-                                                     VLC_OBJECT_DECODER,
-                                                     FIND_PARENT );
-        if( p_dec_obj != NULL )
-        {
-            ppsz_varnames[i] = "ffmpeg-pp-q";
-            pi_objects[i++] = p_dec_obj->i_object_id;
-            vlc_object_release( p_dec_obj );
-        }
-
-        vlc_object_release( p_object );
-    }
-
     /* Input menu */
-    ppsz_varnames[i++] = NULL; /* Separator */
-    ppsz_varnames[i++] = _("Input menu");
-    ppsz_varnames[i++] = NULL; /* Separator */
-
     p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
                                                 FIND_ANYWHERE );
     if( p_object != NULL )
@@ -220,14 +180,71 @@ void PopupMenu( intf_thread_t *p_intf, wxWindow *p_parent,
         pi_objects[i++] = p_object->i_object_id;
         ppsz_varnames[i] = "spu-es";
         pi_objects[i++] = p_object->i_object_id;
+    }
+    p_input = p_object;
+    if( !p_input ) goto interfacemenu;
+
+    /* Video menu */
+    if( i != i_last_separator ) ppsz_varnames[i++] = NULL; /* Separator */
+    i_last_separator = i;
+
+    p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                                FIND_ANYWHERE );
+    if( p_object != NULL )
+    {
+        vlc_object_t *p_dec_obj;
+
+        ppsz_varnames[i] = "fullscreen";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "zoom";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "deinterlace";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "aspect-ratio";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "crop";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "video-on-top";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "directx-wallpaper";
+        pi_objects[i++] = p_object->i_object_id;
+
+        p_dec_obj = (vlc_object_t *)vlc_object_find( p_object,
+                                                     VLC_OBJECT_DECODER,
+                                                     FIND_PARENT );
+        if( p_dec_obj != NULL )
+        {
+            ppsz_varnames[i] = "ffmpeg-pp-q";
+            pi_objects[i++] = p_dec_obj->i_object_id;
+            vlc_object_release( p_dec_obj );
+        }
 
         vlc_object_release( p_object );
     }
 
+    /* Audio menu */
+    if( i != i_last_separator ) ppsz_varnames[i++] = NULL; /* Separator */
+    i_last_separator  = i;
+
+    p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_AOUT,
+                                                FIND_ANYWHERE );
+    if( p_object != NULL )
+    {
+        ppsz_varnames[i] = "audio-device";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "audio-channels";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "visual";
+        pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "equalizer";
+        pi_objects[i++] = p_object->i_object_id;
+        vlc_object_release( p_object );
+    }
+
+ interfacemenu:
     /* Interface menu */
-    ppsz_varnames[i++] = NULL; /* Separator */
-    ppsz_varnames[i++] = _("Interface menu");
-    ppsz_varnames[i++] = NULL; /* Separator */
+    if( i != i_last_separator ) ppsz_varnames[i++] = NULL; /* Separator */
+    i_last_separator = i;
 
     /* vlc_object_find is needed because of the dialogs provider case */
     p_object = (vlc_object_t *)vlc_object_find( p_intf, VLC_OBJECT_INTF,
@@ -238,6 +255,8 @@ void PopupMenu( intf_thread_t *p_intf, wxWindow *p_parent,
         pi_objects[i++] = p_object->i_object_id;
         ppsz_varnames[i] = "intf-add";
         pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "intf-skins";
+        pi_objects[i++] = p_object->i_object_id;
 
         vlc_object_release( p_object );
     }
@@ -247,10 +266,39 @@ void PopupMenu( intf_thread_t *p_intf, wxWindow *p_parent,
     popupmenu.Populate( i, ppsz_varnames, pi_objects );
 
     /* Add static entries */
-    popupmenu.AppendSeparator();
-    popupmenu.Append( MenuDummy_Event, wxU("Open..."),
+    if( p_input != NULL )
+    {
+        vlc_value_t val;
+        popupmenu.InsertSeparator( 0 );
+        popupmenu.Insert( 0, Stop_Event, wxU(_("Stop")) );
+        popupmenu.Insert( 0, Previous_Event, wxU(_("Previous")) );
+        popupmenu.Insert( 0, Next_Event, wxU(_("Next")) );
+
+        var_Get( p_input, "state", &val );
+        if( val.i_int == PAUSE_S )
+            popupmenu.Insert( 0, Play_Event, wxU(_("Play")) );
+        else
+            popupmenu.Insert( 0, Pause_Event, wxU(_("Pause")) );
+
+        vlc_object_release( p_input );
+    }
+    else
+    {
+        playlist_t * p_playlist =
+            (playlist_t *)vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                           FIND_ANYWHERE );
+        if( p_playlist && p_playlist->i_size )
+        {
+            popupmenu.InsertSeparator( 0 );
+            popupmenu.Insert( 0, Play_Event, wxU(_("Play")) );
+        }
+        if( p_playlist ) vlc_object_release( p_playlist );
+    }
+
+    popupmenu.Append( MenuDummy_Event, wxU(_("Miscellaneous")),
+                      MiscMenu( p_intf ), wxT("") );
+    popupmenu.Append( MenuDummy_Event, wxU(_("Open")),
                       OpenStreamMenu( p_intf ), wxT("") );
-    popupmenu.Append( Preferences_Event, wxU(_("&Preferences...")) );
 
     p_intf->p_sys->p_popup_menu = &popupmenu;
     p_parent->PopupMenu( &popupmenu, pos.x, pos.y );
@@ -288,6 +336,8 @@ wxMenu *AudioMenu( intf_thread_t *_p_intf, wxWindow *p_parent, wxMenu *p_menu )
         pi_objects[i++] = p_object->i_object_id;
         ppsz_varnames[i] = "visual";
         pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "equalizer";
+        pi_objects[i++] = p_object->i_object_id;
         vlc_object_release( p_object );
     }
 
@@ -344,6 +394,8 @@ wxMenu *VideoMenu( intf_thread_t *_p_intf, wxWindow *p_parent, wxMenu *p_menu )
         pi_objects[i++] = p_object->i_object_id;
         ppsz_varnames[i] = "video-on-top";
         pi_objects[i++] = p_object->i_object_id;
+        ppsz_varnames[i] = "directx-wallpaper";
+        pi_objects[i++] = p_object->i_object_id;
 
         p_dec_obj = (vlc_object_t *)vlc_object_find( p_object,
                                                      VLC_OBJECT_DECODER,
@@ -372,7 +424,7 @@ wxMenu *VideoMenu( intf_thread_t *_p_intf, wxWindow *p_parent, wxMenu *p_menu )
 
 wxMenu *NavigMenu( intf_thread_t *_p_intf, wxWindow *p_parent, wxMenu *p_menu )
 {
-#define MAX_NAVIG_ITEMS 10
+#define MAX_NAVIG_ITEMS 15
 
     vlc_object_t *p_object;
     char *ppsz_varnames[MAX_NAVIG_ITEMS];
@@ -832,6 +884,12 @@ void MenuEvtHandler::OnShowDialog( wxCommandEvent& event )
         case OpenCapture_Event:
             i_id = INTF_DIALOG_CAPTURE;
             break;
+        case MediaInfo_Event:
+            i_id = INTF_DIALOG_FILEINFO;
+            break;
+        case Messages_Event:
+            i_id = INTF_DIALOG_MESSAGES;
+            break;
         case Preferences_Event:
             i_id = INTF_DIALOG_PREFS;
             break;
@@ -851,6 +909,47 @@ void MenuEvtHandler::OnMenuEvent( wxCommandEvent& event )
     int i_hotkey_event = p_intf->p_sys->i_first_hotkey_event;
     int i_hotkeys = p_intf->p_sys->i_hotkeys;
 
+    if( event.GetId() >= Play_Event && event.GetId() <= Stop_Event )
+    {
+        input_thread_t *p_input;
+        playlist_t * p_playlist =
+            (playlist_t *)vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                           FIND_ANYWHERE );
+        if( !p_playlist ) return;
+
+        switch( event.GetId() )
+        {
+        case Play_Event:
+        case Pause_Event:
+            p_input =
+                (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                                   FIND_ANYWHERE );
+            if( !p_input ) playlist_Play( p_playlist );
+            else
+            {
+                vlc_value_t val;
+                var_Get( p_input, "state", &val );
+                if( val.i_int != PAUSE_S ) val.i_int = PAUSE_S;
+                else val.i_int = PLAYING_S;
+                var_Set( p_input, "state", val );
+                vlc_object_release( p_input );
+            }
+            break;
+        case Stop_Event:
+            playlist_Stop( p_playlist );
+            break;
+        case Previous_Event:
+            playlist_Prev( p_playlist );
+            break;
+        case Next_Event:
+            playlist_Next( p_playlist );
+            break;
+        }
+
+        vlc_object_release( p_playlist );
+        return;
+    }
+
     /* Check if this is an auto generated menu item */
     if( event.GetId() < FirstAutoGenerated_Event )
     {
