@@ -2,17 +2,17 @@
  * x11.c : X11 plugin for vlc
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: x11.c,v 1.5 2003/02/20 01:52:46 sigmunau Exp $
+ * $Id: x11.c 9268 2004-11-10 13:01:48Z gbazin $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
  *          David Kennedy <dkennedy@tinytoad.com>
- *      
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -40,13 +40,13 @@ extern void E_(Deactivate) ( vlc_object_t * );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define ALT_FS_TEXT N_("alternate fullscreen method")
+#define ALT_FS_TEXT N_("Alternate fullscreen method")
 #define ALT_FS_LONGTEXT N_( \
     "There are two ways to make a fullscreen window, unfortunately each one " \
     "has its drawbacks.\n" \
-    "1) Let the window manager handle your fullscreen window (default). But " \
+    "1) Let the window manager handle your fullscreen window (default), but " \
     "things like taskbars will likely show on top of the video.\n" \
-    "2) Completly bypass the window manager, but then nothing will be able " \
+    "2) Completely bypass the window manager, but then nothing will be able " \
     "to show on top of the video.")
 
 #define DISPLAY_TEXT N_("X11 display name")
@@ -54,19 +54,26 @@ extern void E_(Deactivate) ( vlc_object_t * );
     "Specify the X11 hardware display you want to use. By default VLC will " \
     "use the value of the DISPLAY environment variable.")
 
-#define SHM_TEXT N_("use shared memory")
+#define SHM_TEXT N_("Use shared memory")
 #define SHM_LONGTEXT N_( \
     "Use shared memory to communicate between VLC and the X server.")
 
+#define SCREEN_TEXT N_("choose the screen to be used for fullscreen mode.")
+#define SCREEN_LONGTEXT N_( \
+    "Choose the screen you want to use in fullscreen mode. For instance " \
+    "set it to 0 for first screen, 1 for the second.")
+
 vlc_module_begin();
-    add_category_hint( N_("X11"), NULL, VLC_TRUE );
     add_string( "x11-display", NULL, NULL, DISPLAY_TEXT, DISPLAY_LONGTEXT, VLC_TRUE );
     add_bool( "x11-altfullscreen", 0, NULL, ALT_FS_TEXT, ALT_FS_LONGTEXT, VLC_TRUE );
 #ifdef HAVE_SYS_SHM_H
     add_bool( "x11-shm", 1, NULL, SHM_TEXT, SHM_LONGTEXT, VLC_TRUE );
 #endif
-    set_description( _("X11 module") );
-    set_capability( "video output", 50 );
+#ifdef HAVE_XINERAMA
+    add_integer ( "x11-xineramascreen", 0, NULL, SCREEN_TEXT, SCREEN_LONGTEXT, VLC_TRUE );
+#endif
+    set_description( _("X11 video output") );
+    set_capability( "video output", 70 );
     set_callbacks( E_(Activate), E_(Deactivate) );
 vlc_module_end();
 
