@@ -1,8 +1,8 @@
 /*****************************************************************************
  * playlist.c : Playlist management functions
  *****************************************************************************
- * Copyright (C) 1999-2001 VideoLAN
- * $Id: playlist.c,v 1.70 2003/12/13 17:16:11 gbazin Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: playlist.c 8153 2004-07-08 12:25:20Z gbazin $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -29,13 +29,9 @@
 #include <vlc/sout.h>
 #include <vlc/input.h>
 
-#include "stream_control.h"
-#include "input_ext-intf.h"
-
 #include "vlc_playlist.h"
 
 #define PLAYLIST_FILE_HEADER_0_5  "# vlc playlist file version 0.5"
-#define PLAYLIST_FILE_HEADER_0_6  "# vlc playlist file version 0.6"
 
 /*****************************************************************************
  * Local prototypes
@@ -44,10 +40,6 @@ static void RunThread ( playlist_t * );
 static void SkipItem  ( playlist_t *, int );
 static void PlayItem  ( playlist_t * );
 
-#if 0
-static void Poubellize ( playlist_t *, input_thread_t * );
-#endif
-
 /**
  * Create playlist
  *
@@ -72,12 +64,26 @@ playlist_t * __playlist_Create ( vlc_object_t *p_parent )
     val.b_bool = VLC_TRUE;
     var_Set( p_playlist, "intf-change", val );
 
+    var_Create( p_playlist, "item-change", VLC_VAR_INTEGER );
+    val.i_int = -1;
+    var_Set( p_playlist, "item-change", val );
+
+    var_Create( p_playlist, "playlist-current", VLC_VAR_INTEGER );
+    val.i_int = -1;
+    var_Set( p_playlist, "playlist-current", val );
+
     var_Create( p_playlist, "intf-popupmenu", VLC_VAR_BOOL );
 
     var_Create( p_playlist, "intf-show", VLC_VAR_BOOL );
     val.b_bool = VLC_TRUE;
     var_Set( p_playlist, "intf-show", val );
 
+
+    var_Create( p_playlist, "prevent-skip", VLC_VAR_BOOL );
+    val.b_bool = VLC_FALSE;
+    var_Set( p_playlist, "prevent-skip", val );
+
+    var_Create( p_playlist, "play-and-stop", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_playlist, "random", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_playlist, "repeat", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_playlist, "loop", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
@@ -90,9 +96,12 @@ playlist_t * __playlist_Create ( vlc_object_t *p_parent )
 
     p_playlist->i_groups = 0;
     p_playlist->pp_groups = NULL;
-    p_playlist->i_max_id = 0;
+    p_playlist->i_last_group = 0;
+    p_playlist->i_last_id = 0;
+    p_playlist->i_sort = SORT_ID;
+    p_playlist->i_order = ORDER_NORMAL;
 
-    playlist_CreateGroup( p_playlist, "Normal" );
+    playlist_CreateGroup( p_playlist, _("Normal") );
 
     if( vlc_thread_create( p_playlist, "playlist", RunThread,
                            VLC_THREAD_PRIORITY_LOW, VLC_TRUE ) )
@@ -121,6 +130,15 @@ void playlist_Destroy( playlist_t * p_playlist )
     vlc_thread_join( p_playlist );
 
     var_Destroy( p_playlist, "intf-change" );
+    var_Destroy( p_playlist, "item-change" );
+    var_Destroy( p_playlist, "playlist-current" );
+    var_Destroy( p_playlist, "intf-popmenu" );
+    var_Destroy( p_playlist, "intf-show" );
+    var_Destroy( p_playlist, "prevent-skip" );
+    var_Destroy( p_playlist, "play-and-stop" );
+    var_Destroy( p_playlist, "random" );
+    var_Destroy( p_playlist, "repeat" );
+    var_Destroy( p_playlist, "loop" );
 
     while( p_playlist->i_groups > 0 )
     {
@@ -137,19 +155,26 @@ void playlist_Destroy( playlist_t * p_playlist )
 
 
 /**
- * Do a playlist action
+ * Do a playlist action.
  *
+ * If there is something in the playlist then you can do playlist actions.
  * \param p_playlist the playlist to do the command on
  * \param i_command the command to do
  * \param i_arg the argument to the command. See playlist_command_t for details
  */
- void playlist_Command( playlist_t * p_playlist, playlist_command_t i_command,
+void playlist_Command( playlist_t * p_playlist, playlist_command_t i_command,
                        int i_arg )
 {
     vlc_value_t val;
 
     vlc_mutex_lock( &p_playlist->object_lock );
 
+    if( p_playlist->i_size <= 0 )
+    {
+        vlc_mutex_unlock( &p_playlist->object_lock );
+        return;
+    }
+
     switch( i_command )
     {
     case PLAYLIST_STOP:
@@ -157,6 +182,11 @@ void playlist_Destroy( playlist_t * p_playlist )
         if( p_playlist->p_input )
         {
             input_StopThread( p_playlist->p_input );
+            val.i_int = p_playlist->i_index;
+            /* Does not matter if we unlock here */
+            vlc_mutex_unlock( &p_playlist->object_lock );
+            var_Set( p_playlist, "item-change",val );
+            vlc_mutex_lock( &p_playlist->object_lock );
         }
         break;
 
@@ -221,6 +251,8 @@ void playlist_Destroy( playlist_t * p_playlist )
             {
                 input_StopThread( p_playlist->p_input );
             }
+            val.b_bool = VLC_TRUE;
+            var_Set( p_playlist, "prevent-skip", val );
             p_playlist->i_status = PLAYLIST_RUNNING;
         }
         break;
@@ -231,44 +263,39 @@ void playlist_Destroy( playlist_t * p_playlist )
     }
 
     vlc_mutex_unlock( &p_playlist->object_lock );
-
+#if 0
     val.b_bool = VLC_TRUE;
     var_Set( p_playlist, "intf-change", val );
-
+#endif
     return;
 }
-/* Following functions are local */
 
-static void ObjectGarbageCollector( playlist_t *p_playlist,
-                                    int i_type,
-                                    mtime_t *pi_obj_destroyed_date )
+
+static mtime_t ObjectGarbageCollector( playlist_t *p_playlist, int i_type,
+                                       mtime_t destroy_date )
 {
     vlc_object_t *p_obj;
-    if( *pi_obj_destroyed_date > mdate() )
-    {
-        return;
-    }
 
-    if( *pi_obj_destroyed_date == 0 )
+    if( destroy_date > mdate() ) return destroy_date;
+
+    if( destroy_date == 0 )
     {
         /* give a little time */
-        *pi_obj_destroyed_date = mdate() + I64C(300000);
+        return mdate() + I64C(1000000);
     }
     else
     {
-        while( ( p_obj = vlc_object_find( p_playlist,
-                                           i_type,
-                                           FIND_CHILD ) ) )
+        while( ( p_obj = vlc_object_find( p_playlist, i_type, FIND_CHILD ) ) )
         {
             if( p_obj->p_parent != (vlc_object_t*)p_playlist )
             {
-                /* only first chiled (ie unused) */
+                /* only first child (ie unused) */
                 vlc_object_release( p_obj );
                 break;
             }
             if( i_type == VLC_OBJECT_VOUT )
             {
-                msg_Dbg( p_playlist, "vout garbage collector destroying 1 vout" );
+                msg_Dbg( p_playlist, "garbage collector destroying 1 vout" );
                 vlc_object_detach( p_obj );
                 vlc_object_release( p_obj );
                 vout_Destroy( (vout_thread_t *)p_obj );
@@ -279,7 +306,7 @@ static void ObjectGarbageCollector( playlist_t *p_playlist,
                 sout_DeleteInstance( (sout_instance_t*)p_obj );
             }
         }
-        *pi_obj_destroyed_date = 0;
+        return 0;
     }
 }
 
@@ -294,6 +321,8 @@ static void RunThread ( playlist_t *p_playlist )
     mtime_t    i_vout_destroyed_date = 0;
     mtime_t    i_sout_destroyed_date = 0;
 
+    playlist_item_t *p_autodelete_item = 0;
+
     /* Tell above that we're ready */
     vlc_thread_ready( p_playlist );
 
@@ -328,6 +357,14 @@ static void RunThread ( playlist_t *p_playlist )
 
                 i_vout_destroyed_date = 0;
                 i_sout_destroyed_date = 0;
+
+                /* Check for autodeletion */
+                if( p_autodelete_item )
+                {
+                    playlist_ItemDelete( p_autodelete_item );
+                    p_autodelete_item = 0;
+                }
+
                 continue;
             }
             /* This input is dying, let him do */
@@ -339,46 +376,69 @@ static void RunThread ( playlist_t *p_playlist )
             else if( p_playlist->p_input->b_error
                       || p_playlist->p_input->b_eof )
             {
-                /* Check for autodeletion */
+                input_StopThread( p_playlist->p_input );
+
                 if( p_playlist->pp_items[p_playlist->i_index]->b_autodeletion )
                 {
+                    /* This ain't pretty but hey it works */
+                    p_autodelete_item =
+                        p_playlist->pp_items[p_playlist->i_index];
+                    p_playlist->pp_items[p_playlist->i_index] =
+                        playlist_ItemNew( p_playlist,
+                                          p_autodelete_item->input.psz_uri, 0);
+
                     vlc_mutex_unlock( &p_playlist->object_lock );
+                    p_playlist->i_status = PLAYLIST_STOPPED;
                     playlist_Delete( p_playlist, p_playlist->i_index );
-                }
-                else
-                {
-                    /* Select the next playlist item */
-                    SkipItem( p_playlist, 1 );
-                    input_StopThread( p_playlist->p_input );
-                    vlc_mutex_unlock( &p_playlist->object_lock );
+                    p_playlist->i_status = PLAYLIST_RUNNING;
+                    vlc_mutex_lock( &p_playlist->object_lock );
                 }
 
-                val.b_bool = VLC_TRUE;
-                var_Set( p_playlist, "intf-change", val );
+                SkipItem( p_playlist, 1 );
+                vlc_mutex_unlock( &p_playlist->object_lock );
                 continue;
             }
-            else if( p_playlist->p_input->stream.control.i_status != INIT_S )
+            else if( p_playlist->p_input->i_state != INIT_S )
             {
                 vlc_mutex_unlock( &p_playlist->object_lock );
-                ObjectGarbageCollector( p_playlist, VLC_OBJECT_VOUT,
-                                        &i_vout_destroyed_date );
-                ObjectGarbageCollector( p_playlist, VLC_OBJECT_SOUT,
-                                        &i_sout_destroyed_date );
+                i_vout_destroyed_date =
+                    ObjectGarbageCollector( p_playlist, VLC_OBJECT_VOUT,
+                                            i_vout_destroyed_date );
+                i_sout_destroyed_date =
+                    ObjectGarbageCollector( p_playlist, VLC_OBJECT_SOUT,
+                                            i_sout_destroyed_date );
                 vlc_mutex_lock( &p_playlist->object_lock );
             }
         }
         else if( p_playlist->i_status != PLAYLIST_STOPPED )
         {
-            SkipItem( p_playlist, 0 );
-            PlayItem( p_playlist );
+            /* Start another input. Let's check if that item has
+             * been forced. In that case, we override random (by not skipping)
+             * and play-and-stop */
+            vlc_bool_t b_forced;
+            var_Get( p_playlist, "prevent-skip", &val );
+            b_forced = val.b_bool;
+            if( val.b_bool == VLC_FALSE )
+            {
+                SkipItem( p_playlist, 0 );
+            }
+            /* Reset forced status */
+            val.b_bool = VLC_FALSE;
+            var_Set( p_playlist, "prevent-skip", val );
+            /* Check for play-and-stop */
+            var_Get( p_playlist, "play-and-stop", &val );
+            if( val.b_bool == VLC_FALSE || b_forced == VLC_TRUE )
+            {
+                PlayItem( p_playlist );
+            }
         }
         else if( p_playlist->i_status == PLAYLIST_STOPPED )
         {
             vlc_mutex_unlock( &p_playlist->object_lock );
-            ObjectGarbageCollector( p_playlist, VLC_OBJECT_SOUT,
-                                    &i_sout_destroyed_date );
-            ObjectGarbageCollector( p_playlist, VLC_OBJECT_VOUT,
-                                    &i_vout_destroyed_date );
+            i_sout_destroyed_date =
+                ObjectGarbageCollector( p_playlist, VLC_OBJECT_SOUT, mdate() );
+            i_vout_destroyed_date =
+                ObjectGarbageCollector( p_playlist, VLC_OBJECT_VOUT, mdate() );
             vlc_mutex_lock( &p_playlist->object_lock );
         }
         vlc_mutex_unlock( &p_playlist->object_lock );
@@ -466,6 +526,7 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
     int i_oldindex = p_playlist->i_index;
     vlc_bool_t b_random, b_repeat, b_loop;
     vlc_value_t val;
+    int i_count;
 
     /* If the playlist is empty, there is no current item */
     if( p_playlist->i_size == 0 )
@@ -485,27 +546,38 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
     if( b_random )
     {
         srand( (unsigned int)mdate() );
-
-        /* Simple random stuff - we cheat a bit to minimize the chances to
-         * get the same index again. */
-        i_arg = (int)((float)p_playlist->i_size * rand() / (RAND_MAX+1.0));
-        if( i_arg == 0 )
+        i_count = 0;
+        while( i_count < p_playlist->i_size )
         {
-            i_arg = (int)((float)p_playlist->i_size * rand() / (RAND_MAX+1.0));
+            p_playlist->i_index =
+                (int)((float)p_playlist->i_size * rand() / (RAND_MAX+1.0));
+            /* Check if the item has not already been played */
+            if( p_playlist->pp_items[p_playlist->i_index]->i_nb_played == 0 )
+                break;
+            i_count++;
+        }
+        if( i_count == p_playlist->i_size )
+        {
+            /* The whole playlist has been played: reset the counters */
+            while( i_count > 0 )
+            {
+                p_playlist->pp_items[--i_count]->i_nb_played = 0;
+            }
+            if( !b_loop )
+            {
+                p_playlist->i_status = PLAYLIST_STOPPED;
+            }
         }
     }
-    if( b_repeat )
+    else if( !b_repeat )
     {
-        i_arg = 0;
+        p_playlist->i_index += i_arg;
     }
-    p_playlist->i_index += i_arg;
 
     /* Boundary check */
     if( p_playlist->i_index >= p_playlist->i_size )
     {
-        if( p_playlist->i_status == PLAYLIST_STOPPED
-             || b_random
-             || b_loop )
+        if( p_playlist->i_status == PLAYLIST_STOPPED || b_random || b_loop )
         {
             p_playlist->i_index -= p_playlist->i_size
                          * ( p_playlist->i_index / p_playlist->i_size );
@@ -523,9 +595,9 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
     }
 
     /* Check that the item is enabled */
-   if( p_playlist->pp_items[p_playlist->i_index]->b_enabled == VLC_FALSE &&
-       p_playlist->i_enabled != 0)
-   {
+    if( p_playlist->pp_items[p_playlist->i_index]->b_enabled == VLC_FALSE &&
+        p_playlist->i_enabled != 0)
+    {
         SkipItem( p_playlist , 1 );
     }
 }
@@ -538,22 +610,30 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
  *****************************************************************************/
 static void PlayItem( playlist_t *p_playlist )
 {
+    playlist_item_t *p_item;
+    vlc_value_t val;
     if( p_playlist->i_index == -1 )
     {
         if( p_playlist->i_size == 0 || p_playlist->i_enabled == 0)
         {
             return;
         }
-
         SkipItem( p_playlist, 1 );
     }
-
     if( p_playlist->i_enabled == 0)
     {
         return;
     }
 
     msg_Dbg( p_playlist, "creating new input thread" );
-    p_playlist->p_input = input_CreateThread( p_playlist,
-                                  p_playlist->pp_items[p_playlist->i_index] );
+    p_item = p_playlist->pp_items[p_playlist->i_index];
+
+    p_item->i_nb_played++;
+    p_playlist->p_input = input_CreateThread( p_playlist, &p_item->input );
+
+    val.i_int = p_playlist->i_index;
+    /* unlock the playlist to set the var...mmm */
+    vlc_mutex_unlock( &p_playlist->object_lock);
+    var_Set( p_playlist, "playlist-current", val);
+    vlc_mutex_lock( &p_playlist->object_lock);
 }
