@@ -1,8 +1,8 @@
 /*****************************************************************************
  * video_text.c : text manipulation functions
  *****************************************************************************
- * Copyright (C) 1999-2001 VideoLAN
- * $Id: video_text.c,v 1.49 2003/12/22 14:32:57 sam Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: video_text.c 8722 2004-09-17 14:47:01Z gbazin $
  *
  * Author: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -21,11 +21,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 #include <vlc/vout.h>
-#include <osd.h>
+#include "vlc_block.h"
+#include "vlc_filter.h"
+#include "osd.h"
 
 /**
  * \brief Show text on the video for some time
  * \param p_vout pointer to the vout the text is to be showed on
+ * \param i_channel Subpicture channel
  * \param psz_string The text to be shown
  * \param p_style Pointer to a struct with text style info
  * \param i_flags flags for alignment and such
@@ -33,30 +36,22 @@
  * \param i_vmargin vertical margin in pixels
  * \param i_duration Amount of time the text is to be shown.
  */
-subpicture_t *vout_ShowTextRelative( vout_thread_t *p_vout, char *psz_string,
-                              text_style_t *p_style, int i_flags,
-                              int i_hmargin, int i_vmargin,
-                              mtime_t i_duration )
+int vout_ShowTextRelative( vout_thread_t *p_vout, int i_channel,
+                           char *psz_string, text_style_t *p_style,
+                           int i_flags, int i_hmargin, int i_vmargin,
+                           mtime_t i_duration )
 {
-    subpicture_t *p_subpic = NULL;
     mtime_t i_now = mdate();
 
-    if ( p_vout->pf_add_string )
-    {
-        p_subpic = p_vout->pf_add_string( p_vout, psz_string, p_style, i_flags,
-                             i_hmargin, i_vmargin, i_now, i_now + i_duration );
-    }
-    else
-    {
-        msg_Warn( p_vout, "No text renderer found" );
-    }
-
-    return p_subpic;
+    return vout_ShowTextAbsolute( p_vout, i_channel, psz_string,
+                                  p_style, i_flags, i_hmargin, i_vmargin,
+                                  i_now, i_now + i_duration );
 }
 
 /**
  * \brief Show text on the video from a given start date to a given end date
  * \param p_vout pointer to the vout the text is to be showed on
+ * \param i_channel Subpicture channel
  * \param psz_string The text to be shown
  * \param p_style Pointer to a struct with text style info
  * \param i_flags flags for alignment and such
@@ -67,20 +62,47 @@ subpicture_t *vout_ShowTextRelative( vout_thread_t *p_vout, char *psz_string,
  *               if this is 0 the string will be shown untill the next string
  *               is about to be shown
  */
-void vout_ShowTextAbsolute( vout_thread_t *p_vout, char *psz_string,
-                              text_style_t *p_style, int i_flags,
-                              int i_hmargin, int i_vmargin, mtime_t i_start,
-                              mtime_t i_stop )
+int vout_ShowTextAbsolute( vout_thread_t *p_vout, int i_channel,
+                           char *psz_string, text_style_t *p_style,
+                           int i_flags, int i_hmargin, int i_vmargin,
+                           mtime_t i_start, mtime_t i_stop )
 {
-    if ( p_vout->pf_add_string )
+    subpicture_t *p_spu;
+    video_format_t fmt;
+
+    if( !psz_string ) return VLC_EGENERIC;
+
+    p_spu = spu_CreateSubpicture( p_vout->p_spu );
+    if( !p_spu ) return VLC_EGENERIC;
+
+    /* Create a new subpicture region */
+    memset( &fmt, 0, sizeof(video_format_t) );
+    fmt.i_chroma = VLC_FOURCC('T','E','X','T');
+    fmt.i_aspect = 0;
+    fmt.i_width = fmt.i_height = 0;
+    fmt.i_x_offset = fmt.i_y_offset = 0;
+    p_spu->p_region = p_spu->pf_create_region( VLC_OBJECT(p_vout), &fmt );
+    if( !p_spu->p_region )
     {
-        p_vout->pf_add_string( p_vout, psz_string, p_style, i_flags, i_hmargin,
-                               i_vmargin, i_start, i_stop );
-    }
-    else
-    {
-        msg_Warn( p_vout, "No text renderer found" );
+        msg_Err( p_vout, "cannot allocate SPU region" );
+        spu_DestroySubpicture( p_vout->p_spu, p_spu );
+        return VLC_EGENERIC;
     }
+
+    p_spu->p_region->psz_text = strdup( psz_string );
+    p_spu->i_start = i_start;
+    p_spu->i_stop = i_stop;
+    p_spu->b_ephemer = VLC_TRUE;
+    p_spu->b_absolute = VLC_FALSE;
+
+    p_spu->i_x = i_hmargin;
+    p_spu->i_y = i_vmargin;
+    p_spu->i_flags = i_flags;
+    p_spu->i_channel = i_channel;
+
+    spu_DisplaySubpicture( p_vout->p_spu, p_spu );
+
+    return VLC_SUCCESS;
 }
 
 
@@ -88,11 +110,15 @@ void vout_ShowTextAbsolute( vout_thread_t *p_vout, char *psz_string,
  * \brief Write an informative message at the default location,
  *        for the default duration and only if the OSD option is enabled.
  * \param p_caller The object that called the function.
- * \param psz_string The text to be shown
+ * \param i_channel Subpicture channel
+ * \param psz_format printf style formatting
  **/
-void vout_OSDMessage( vlc_object_t *p_caller, char *psz_string )
+void __vout_OSDMessage( vlc_object_t *p_caller, int i_channel,
+                        char *psz_format, ... )
 {
     vout_thread_t *p_vout;
+    char *psz_string;
+    va_list args;
 
     if( !config_GetInt( p_caller, "osd" ) ) return;
 
@@ -100,19 +126,14 @@ void vout_OSDMessage( vlc_object_t *p_caller, char *psz_string )
 
     if( p_vout )
     {
-        vlc_mutex_lock( &p_vout->change_lock );
+        va_start( args, psz_format );
+        vasprintf( &psz_string, psz_format, args );
 
-        if( p_vout->p_last_osd_message )
-        {
-            vout_DestroySubPicture( p_vout, p_vout->p_last_osd_message );
-        }
-
-        p_vout->p_last_osd_message = vout_ShowTextRelative( p_vout, psz_string,
-                        NULL, OSD_ALIGN_TOP|OSD_ALIGN_RIGHT, 30,20,1000000 );
-
-        vlc_mutex_unlock( &p_vout->change_lock );
+        vout_ShowTextRelative( p_vout, i_channel, psz_string, NULL,
+                               OSD_ALIGN_TOP|OSD_ALIGN_RIGHT, 30,20,1000000 );
 
         vlc_object_release( p_vout );
+        free( psz_string );
+        va_end( args );
     }
 }
-
